from django.conf.urls.defaults import patterns, url
from tendenci.apps.pages.feeds import LatestEntriesFeed

urlpatterns = patterns('tendenci.apps.pages.views',
    url(r'^$', 'index', name="pages"),
    url(r'^search/$', 'search', name="page.search"),
    url(r'^print-view/(?P<slug>[\w\-\/]+)/$', 'print_view', name="page.print_view"),
    url(r'^export/$', 'export', name="page.export"),
    url(r'^preview/$', 'preview', name="page.preview"),
    url(r'^preview/(?P<id>\d+)/$', 'preview', name="page.preview"),
    url(r'^add/$', 'add', name="page.add"),
    url(r'^edit/(?P<id>\d+)/$', 'edit', name="page.edit"),
    url(r'^versions/(?P<hash>[\w\-]+)/$', 'index', name="page.version"),
    url(r'^inactive/(?P<id>\d+)/$', 'index', name="page.inactive"),
    url(r'^edit/meta/(?P<id>\d+)/$', 'edit_meta', name="page.edit.meta"),
    url(r'^delete/(?P<id>\d+)/$', 'delete', name="page.delete"),
    url(r'^feed/$', LatestEntriesFeed(), name='page.feed'),
    url(r'^header_image/(?P<id>\d+)/$', 'display_header_image', name="page.header_image"),
)



from tendenci.apps.contacts.models import Contact
from django import forms
from captcha.fields import CaptchaField
from django.utils.translation import ugettext_lazy as _ 

from tendenci.core.base.fields import EmailVerificationField

class ContactForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = (
            'first_name',
            'last_name',
            'addresses',
            'phones',
            'emails',
            'urls',
            'companies',
            'message',
        )

class SubmitContactForm(forms.Form):

    first_name = forms.CharField(max_length=100)
    last_name = forms.CharField(max_length=100, required=False)

    address = forms.CharField(max_length=50, required=False)
    city = forms.CharField(max_length=50, required=False)
    state = forms.CharField(max_length=50, required=False)
    zipcode = forms.CharField(max_length=10, required=False)
    country = forms.CharField(max_length=100, required=False)

    phone = forms.CharField(max_length=20, required=False)
    email = EmailVerificationField(label=_("Email"))
    url = forms.URLField(label=_('URL'), max_length=200, required=False)

    message = forms.CharField(widget=forms.Textarea)
    captcha = CaptchaField()



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    Set the media files permissions on S3.

    Example:
        ./manage.py set_media_files_perms
    """

    def handle(self, *apps, **kwargs):
        from tendenci.core.files.models import File as tFile
        from tendenci.libs.boto_s3.utils import set_s3_file_permission

        files = tFile.objects.all()
        count = 0
        for tfile in files:
            if tfile.file:
                if tfile.allow_anonymous_view:
                    perm = 'public'
                else:
                    perm = 'private'

                print 'Setting %s to %s' % (tfile.file.name,
                                            perm)

                set_s3_file_permission(tfile.file.name,
                                       public=(perm == 'public'))
                count += 1

        print 'Done'
        print 'Total files processed', count



from django.utils.translation import ugettext_lazy as _
from django.utils.html import mark_safe

from model_report.report import reports, ReportAdmin
from model_report.utils import (sum_column, us_date_format, date_label, obj_type_format)

from tendenci.apps.invoices.models import Invoice
from tendenci.core.site_settings.utils import get_setting

    
def id_format(value, instance):
    invoice = Invoice.objects.get(id=value)
    link = invoice.get_absolute_url()
    html = "<a href=\"%s\">%s</a>" % (link, value)
    return mark_safe(html)

def currency_format(value, instance):
    currencysymbol = get_setting("site", "global", "currencysymbol")
    return "%s%s" % (currencysymbol, value)

class InvoiceReport(ReportAdmin):
    # choose a title for your report for h1, title tag and report list
    title = _('Invoice Report')
    
    # specify your model
    model = Invoice
    
    # fields in the specified model to display in the report table
    fields = [
        'id',
        'bill_to',
        'create_dt',
        'status_detail',
        'object_type',
        'payments_credits',
        'balance',
        'total'
    ]
    
    # fields in the model to show filters for
    list_filter = ('status_detail', 'create_dt',)
    
    # fields in the model to order results by
    list_order_by = ('create_dt', 'status_detail')
    
    # fields to group results by
    list_group_by = ('object_type',)
    
    # allowed export formats. default is excel and pdf
    exports = ('excel', 'pdf',)
    
    # type = report for report only, type = chart for report and charts. default is report.
    type = 'report'
    
    # override field formats by referencing a function
    override_field_formats = {
        'create_dt': us_date_format,
        'object_type': obj_type_format,
        'id': id_format,
        'balance': currency_format,
        'total': currency_format,
        'payments_credits': currency_format
    }
    
    # override the label for a field by referencing a function
    override_field_labels = {
        'create_dt': date_label
    }
    
    group_totals = {
        'balance': sum_column,
        'total': sum_column,
        'payments_credits': sum_column
    }
    
    report_totals = {
        'balance': sum_column,
        'total': sum_column,
        'payments_credits': sum_column
    }

# register your report with the slug and name
reports.register('invoices', InvoiceReport)



def payment_update_stripe(request, charge_response, payment):
    
    if not isinstance(charge_response, str) and charge_response.paid:
        payment.status_detail = 'approved'
        payment.response_code = '1'
        payment.response_subcode = '1'
        payment.response_reason_code = '1'
        payment.response_reason_text = 'This transaction has been approved. (Created# %s)' % charge_response.created
        payment.trans_id = charge_response.id
    else:
        payment.response_code = 0
        payment.response_reason_code = 0
        payment.response_reason_text = charge_response
        
    if payment.is_approved:
        payment.mark_as_paid()
        payment.save()
        payment.invoice.make_payment(request.user, payment.amount)            
    else:
        if payment.status_detail == '':
            payment.status_detail = 'not approved'
        payment.save()


'''
Created on 28-02-2011

@author: hpolloni
'''
import subprocess

from django.contrib.sites.models import get_current_site
from django.core.paginator import EmptyPage, PageNotAnInteger
from django.http import Http404
from django.template.response import TemplateResponse

from django.conf import settings
from django.core.cache import cache
from tendenci.core.sitemaps import TendenciSitemap


_sitemap_cache = []
def get_all_sitemaps():
    for app in settings.INSTALLED_APPS:
        _try_import(app + '.feeds')
    return TendenciSitemap.__subclasses__()


def _try_import(module):
    try:
        __import__(module)
    except ImportError, e:
        pass


def create_sitemap(request):
    sitemap_classes = get_all_sitemaps()
    sitemaps = dict([(cls.__name__, cls) for cls in sitemap_classes])
    return sitemap(request, sitemaps)


def sitemap(request, sitemaps, section=None,
            template_name='sitemap.xml', mimetype='application/xml'):
    req_protocol = 'https' if request.is_secure() else 'http'
    req_site = get_current_site(request)

    if section is not None:
        if section not in sitemaps:
            raise Http404("No sitemap available for section: %r" % section)
        maps = [sitemaps[section]]
    else:
        maps = sitemaps.values()
    page = request.GET.get("p", 1)

    urls = []
    cached = False
    for site in maps:
        try:
            if callable(site):
                site = site()
                site_key = site.__class__.__name__
            else:
                site_key = site.__name__
            # Cache the sitemap urls
            sitemap_cache_key = '.'.join([settings.SITE_CACHE_KEY, 'sitemap_cache', site_key])
            site_urls = cache.get(sitemap_cache_key)
            if not isinstance(site_urls, list):
                if not cached:
                    subprocess.Popen(['python', 'manage.py', 'sitemap_cache'])
                    cached = True
                site_urls = site.get_urls(page=page, site=req_site,
                                          protocol=req_protocol)
                cache.set(sitemap_cache_key, list(site_urls), 86400)
            urls.extend(site_urls)
        except EmptyPage:
            raise Http404("Page %s empty" % page)
        except PageNotAnInteger:
            raise Http404("No page '%s'" % page)
    return TemplateResponse(request, template_name, {'urlset': urls},
                            content_type=mimetype)



from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import AnonymousUser

from tendenci.apps.contributions.models import Contribution


def save_contribution(sender, **kwargs):
    instance = kwargs['instance']

    # TODO: Possibly allow anonymous contributions
    # to be added to the system
    if isinstance(instance.owner, AnonymousUser):
        return None
    if isinstance(instance.creator, AnonymousUser):
        return None
    if not instance.owner or not instance.creator:
        return None

    content_type = ContentType.objects.get_for_model(instance)

    try:  # record exists
        contrib = Contribution.objects.get(
            content_type=content_type,
            object_id=instance.pk
            )
    except:  # create record
        contrib = Contribution()
        contrib.content_type = content_type
        contrib.object_id = instance.id
        contrib.creator = instance.owner

        title_attrs = (
            'title',
            'headline',
            'name',
        )
        for attr in title_attrs:
            if hasattr(instance, attr):
                contrib.title = getattr(instance, attr, '')
                break

        contrib.owner = instance.owner
        contrib.save()


from tendenci.core.registry import site
from tendenci.core.registry.base import LogRegistry, lazy_reverse
from tendenci.addons.tendenci_guide.models import Guide

class GuideRegistry(LogRegistry):
    event_logs = {
        'tendenci_guide':{
            'edit':('1002000', 'A20900'), # base
            'add':('1002100', '119933'), # add
            'edit':('1002200', 'EEDD55'), # edit
            'delete':('1002300', 'AA2222'), # delete
            'search':('1002400', 'CC55EE'), # search
            'detail':('1002500', '55AACC'), # detail
        }
    }

site.register(Guide, GuideRegistry)



from django.template import Library

register = Library()

@register.inclusion_tag("entities/options.html", takes_context=True)
def entity_options(context, user, entity):
    context.update({
        "opt_object": entity,
        "user": user
    })
    return context

@register.inclusion_tag("entities/nav.html", takes_context=True)
def entity_nav(context, user, entity=None):
    context.update({
        "nav_object": entity,
        "user": user
    })
    return context


@register.inclusion_tag("entities/search-form.html", takes_context=True)
def entity_search(context):
    return context


from django import forms
from django.utils.safestring import mark_safe

from widgets import TypeExpMethodWidget, NoticeTimeTypeWidget
from tendenci.core.site_settings.utils import get_setting

        

class TypeExpMethodField(forms.MultiValueField):
    def __init__(self, required=True, widget=TypeExpMethodWidget(attrs=None, fields_pos_d=None),
                label=None, initial=None, help_text=None):
        myfields = ()
        super(TypeExpMethodField, self).__init__(myfields, required, widget,
                                          label, initial, help_text)
        
    def clean(self, value):
        return self.compress(value) 
        
    def compress(self, data_list):
        for i in range(0, len(data_list)):
            if type(data_list[i]) is bool:
                if data_list[i] == False:
                    data_list[i] = ''
                else:
                    data_list[i] = '1'
            if data_list[i] == None:
                data_list[i] = ''
        
        if data_list:
            return ','.join(data_list)
        return None
    
class NoticeTimeTypeField(forms.MultiValueField):
    def __init__(self, required=True, widget=NoticeTimeTypeWidget(attrs=None),
                label=None, initial=None, help_text=None):
        myfields = ()
        super(NoticeTimeTypeField, self).__init__(myfields, required, widget,
                                          label, initial, help_text)
        
    def clean(self, value):
        return self.compress(value) 
        
    def compress(self, data_list):
        if data_list:
            return ','.join(data_list)
        return None
  
    
class PriceInput(forms.TextInput):
    def render(self, name, value, attrs=None):
        currency_symbol = get_setting('site', 'global', 'currencysymbol')
        if currency_symbol == '': currency_symbol = "$"
        return mark_safe('$ %s' % super(PriceInput, self).render(name, value, attrs))
        



from django.contrib import admin

from tendenci.apps.user_groups.models import Group, GroupMembership
from tendenci.apps.user_groups.forms import GroupAdminForm
from tendenci.core.perms.admin import TendenciBaseModelAdmin


class GroupAdmin(TendenciBaseModelAdmin):
    list_display = ['name', 'label', 'show_as_option', 'allow_self_add', 'allow_self_remove', 'admin_status']
    search_fields = ['name', 'label']
    list_filter = ['status']
    fieldsets = (
        (None, {'fields': ('name', 'label', 'entity', 'email_recipient', 'permissions')}),
        ('Flags', {'fields': (
            'show_as_option', 'allow_self_add', 'allow_self_remove')}),
        ('Administrative', {'fields': (
            'allow_anonymous_view', 'user_perms', 'member_perms', 'group_perms', 'status', 'status_detail')}),
    )
    form = GroupAdminForm


class GroupMembershipAdmin(admin.ModelAdmin):
    list_display = ['group', 'member']
    search_fields = ['group', 'member']
    fieldsets = (
        (None, {'fields': ('group', 'member')}),
    )

admin.site.register(Group, GroupAdmin)
#admin.site.register(GroupMembership, GroupMembershipAdmin)


from django.forms.widgets import MultiWidget, DateInput, TextInput
from time import strftime

class SplitDateTimeWidget(MultiWidget):
    def __init__(self, attrs={}, date_format=None, time_format=None):
        if 'date_class' in attrs:
            date_class = attrs['date_class']
            del attrs['date_class']
        else:
            date_class = 'datepicker'
        
        if 'time_class' in attrs:
            time_class = attrs['time_class'] 
            del attrs['time_class']
        else:
            time_class = 'timepicker'
            
        time_attrs = attrs.copy()
        time_attrs['class'] = time_class
        date_attrs = attrs.copy()
        date_attrs['class'] = date_class

        widgets = (DateInput(attrs=date_attrs, format=date_format),
                   TextInput(attrs=time_attrs))

        super(SplitDateTimeWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            date = strftime("%Y-%m-%d", value.timetuple())
            time = strftime("%I:%M %p", value.timetuple())
            return (date, time)
        else:
            return (None, None)

    def format_output(self, rendered_widgets):
        """
        Given a list of rendered widgets (as strings), it inserts an HTML
        linebreak between them.

        Returns a Unicode string representing the HTML for the whole lot.
        """
        return "%s&nbsp;%s" % (rendered_widgets[0], rendered_widgets[1])


class EmailVerificationWidget(MultiWidget):
    def __init__(self, attrs={}):
        if 'email_class_0' in attrs:
            email_class_0 = attrs['email_class_0']
            del attrs['email_class_0']
        else:
            email_class_0 = 'email-verification-0'
        
        if 'email_class_1' in attrs:
            email_class_1 = attrs['email_class_1'] 
            del attrs['email_class_1']
        else:
            email_class_1 = 'email-verification-1'
            
        email0_attrs = attrs.copy()
        email0_attrs['class'] = email_class_0
        email1_attrs = attrs.copy()
        email1_attrs['class'] = email_class_1

        widgets = (TextInput(attrs=email0_attrs),
                   TextInput(attrs=email1_attrs))

        super(EmailVerificationWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            email = value
            return (email, email)
        else:
            return (None, None)

    def format_output(self, rendered_widgets):
        """
        Given a list of rendered widgets (as strings), it inserts an HTML
        linebreak between them.

        Returns a Unicode string representing the HTML for the whole lot.
        """
        label = "<label generated='true' style='display:none; color:red; margin-left: 5px;' class='email-verfication-error'>Please enter similar email addresses.</label>"
        return "%s%s<br>%s" % (rendered_widgets[0], label, rendered_widgets[1])



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Robot'
        db.create_table('robots_robot', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('version', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('status_detail', self.gf('django.db.models.fields.CharField')(default='active', max_length=50)),
        ))
        db.send_create_signal('robots', ['Robot'])


    def backwards(self, orm):
        
        # Deleting model 'Robot'
        db.delete_table('robots_robot')


    models = {
        'robots.robot': {
            'Meta': {'object_name': 'Robot'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['robots']



from django import forms
from widgets import NoNameTextInput
from tendenci.core.payments.models import Payment
#from form_utils.forms import BetterModelForm


class StripeCardForm(forms.Form):
    card_number = forms.CharField(required=False, max_length=20,
                                  widget=NoNameTextInput())
    card_cvc = forms.CharField(required=False, max_length=4,
                               widget=NoNameTextInput())
    card_expiry_month = forms.CharField(required=False, max_length=2,
                                widget=NoNameTextInput())
    card_expiry_year = forms.CharField(required=False, max_length=4,
                                widget=NoNameTextInput())
    
    stripe_token = forms.CharField(required=True, widget=forms.HiddenInput())
    
    def __init__(self, *args, **kwargs):
        super(StripeCardForm, self).__init__(*args, **kwargs)
        self.fields['card_number'].widget.attrs.update({'autocomplete': 'off'})
        self.fields['card_cvc'].label = "Card Code"
        self.fields['card_cvc'].help_text = """
                <a onclick="javascript:return PopupLink(this);"
                target="_blank" href="%s">What's this?</a>
                """ % 'https://account.authorize.net/help/Miscellaneous/Pop-up_Terms/hosted/Card_Code.htm'
        self.fields['card_cvc'].widget.attrs.update({'autocomplete': 'off',
                                                     'size': '4'})
        self.fields['card_expiry_month'].widget.attrs.update({'size': '2'})
        self.fields['card_expiry_year'].widget.attrs.update({'size': '4'})
        
        
class BillingInfoForm(forms.ModelForm):
    class Meta:
        model = Payment
        fields = ('first_name',
                  'last_name', 
                  'email',
                  'company',
                  'address',
                  'address2',
                  'city',
                  'state',
                  'zip',
                  'country',
                  'phone',
                  'fax',
                  )
        
    def __init__(self, *args, **kwargs):
        super(BillingInfoForm, self).__init__(*args, **kwargs)
        self.fields['first_name'].widget.attrs.update({'size': '30'})
        self.fields['last_name'].widget.attrs.update({'size': '30'})
        self.fields['email'].widget.attrs.update({'size': '30'})
        self.fields['company'].widget.attrs.update({'size': '30'})
        self.fields['address'].widget.attrs.update({'size': '30'})
        self.fields['address2'].widget.attrs.update({'size': '30'})
        self.fields['city'].widget.attrs.update({'size': '30'})
        self.fields['country'].widget.attrs.update({'size': '30'})
        self.fields['state'].widget.attrs.update({'size': '10'})
        self.fields['zip'].widget.attrs.update({'size': '10'})
        self.fields['phone'].widget.attrs.update({'size': '10'})
        self.fields['fax'].widget.attrs.update({'size': '10'})
        


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'Import.failure_reason'
        db.add_column('imports_import', 'failure_reason', self.gf('django.db.models.fields.CharField')(default='', max_length=250, blank=True), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'Import.failure_reason'
        db.delete_column('imports_import', 'failure_reason')


    models = {
        'imports.import': {
            'Meta': {'object_name': 'Import'},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_done': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'failure_reason': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '260'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'pending'", 'max_length': '50'}),
            'total_created': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'total_invalid': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'total_updated': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['imports']



import sys
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import Group, Permission, User
from django.core.exceptions import ObjectDoesNotExist


class Command(BaseCommand):
    """
    Downgrades a user to just a regular user

    This command does the following to the user account:
        * Removes them from all groups
        * Removes all user level permissions
        * Sets is_staff to 0
        * Sets is_superuser to 0
        * Sets is_active to 1
        * Removes them from all tendenci user_groups
    """
    option_list = BaseCommand.option_list + (
        make_option('--username',
            action='store',
            dest='username',
            default=None,
            help='Username of the user account being downgraded'),
        )

    def handle(self, *args, **options):
        from tendenci.apps.user_groups.models import GroupMembership

        verbosity = options['verbosity']
        username = options['username']

        if not username:
            raise CommandError('downgrade_user: --username parameter is required')

        # get the user
        try:
            u = User.objects.get(username=username)
        except ObjectDoesNotExist:
            print 'User with username (%s) could not be found' % username
            return

        # Remove the user from all groups
        u.groups.clear()

        # Remove all user-level permissions
        u.user_permissions.clear()

        # Reset permission bits
        u.is_staff = False
        u.is_superuser = False
        u.is_active = True
        u.save()

        # Remove the tendenci group permissions
        group_memberships = GroupMembership.objects.filter(member=u)
        for m in group_memberships:
            m.delete()

        if verbosity >= 2:
            print 'Done downgrading user (%s).' % u



import re

from django.template import RequestContext, Context, Template

def detect_template_tags(string):
    """
        Used to detect wether or not any string contains
        template tags in the system
        returns boolean
    """
    import re
    p = re.compile('{[#{%][^#}%]+[%}#]}', re.IGNORECASE)
    return p.search(string)   

def render_content(content, arg=None, limit=None, image_preview=True):
    """
        checks for template tags within the content and rendered them
        currently it is only rendering streams
    """
    match = detect_template_tags(content)

    if match:
        p = re.compile('{%([\w\s\=]+)%}')
        tags = list(set(re.findall(p, content)))
        TEMPLATE_TAGS = ['{% load box_tags %}']
        if tags: 
            for tag in tags:
                tag = "{%"+tag+"%}"
                context = Context({'user':None})
                t = Template(''.join(TEMPLATE_TAGS) + tag)
                rendered_tag = t.render(context)
                content = content.replace(tag, rendered_tag)  
    return content


import re
import urllib2
from xml.etree import ElementTree as ET

from django.conf import settings
from tendenci.addons.recurring_payments.authnet.utils import to_camel_case

BILL_TO_FIELDS = ('firstName', 'lastName', 'company', 'address', 'city', 'state', 'zip',
                  'country', 'phoneNumber', 'faxNumber')
CREDIT_CARD_FIELDS = ('cardNumber', 'expirationDate', 'cardCode')
TAX_FIELDS = ('amount', 'name', 'description')
SHIPPING_FIELDS = ('amount', 'name', 'description')
LINE_ITEMS_FIELDS = ('itemId', 'name', 'description', 'quantity', 'unitPrice', 'taxable')
ORDER_FIELDS = ('invoiceNumber', 'description', 'purchaseOrderNumber')
TRANSACTION_EXTRA_OPTIONS = {'x_delim_data': 'TRUE', 
                            'x_delim_char': '||', 
                            'x_encap_char': ''}


class CIMBase(object):
    """
    A base class to handle basic CIM request and response
    """
    def __init__(self):
        if hasattr(settings, 'AUTHNET_CIM_TEST_MODE') and  settings.AUTHNET_CIM_TEST_MODE:
            self.cim_url = settings.AUTHNET_CIM_API_TEST_URL
        else:
            self.cim_url = settings.AUTHNET_CIM_API_URL

    def create_base_xml(self, root_name,  **kwargs):
        """
        Create a basic xml file with the merchantAuthentication node.
        """
        root = ET.Element(root_name)
        root.set('xmlns', 'AnetApi/xml/v1/schema/AnetApiSchema.xsd')
        authentication = ET.SubElement(root, 'merchantAuthentication')
        name_e = ET.SubElement(authentication, 'name')
        name_e.text = settings.MERCHANT_LOGIN
        key_e = ET.SubElement(authentication, 'transactionKey')
        key_e.text = settings.MERCHANT_TXN_KEY

        return root
    
    def create_payment_profile_node(self, node_name, billing_info, credit_card_info,  **kwargs):
        # make payment_profiles node
        payment_profiles_node =  ET.Element(node_name)
        # bill_to
        billing_info = to_camel_case(billing_info)
        if billing_info and (type(billing_info) is dict):
            bill_to_node = ET.SubElement(payment_profiles_node, 'billTo')
            bill_to_node = self.build_node_from_dict(bill_to_node, 
                                                     billing_info, 
                                                     BILL_TO_FIELDS)
            
        # credit_card
        payment_node =  ET.SubElement(payment_profiles_node, 'payment')
        credit_card_info= to_camel_case(credit_card_info)
        if credit_card_info and (type(credit_card_info) is dict):
            credit_card_node = ET.SubElement(payment_node, 'creditCard')
            
            credit_card_node = self.build_node_from_dict(credit_card_node, 
                                                         credit_card_info, 
                                                         CREDIT_CARD_FIELDS)
            
        return payment_profiles_node
    
    def build_node_from_dict(self, parent_node, d, field_scope=None):
        for key in d.keys():
            if not field_scope or key in field_scope:
                value = d.get(key)
                node = ET.SubElement(parent_node, key)
                node.text = value
        return parent_node
                
            
        
    
    def process_request(self, xml_root):
        request_xml_str = '%s\n%s' % ('<?xml version="1.0" encoding="utf-8"?>', ET.tostring(xml_root))
        #print request_xml_str
        request = urllib2.Request(self.cim_url, 
                                request_xml_str, 
                                {'Content-Type': 'text/xml',
                                'encoding': 'utf-8'})
        response = urllib2.urlopen(request)
        data = response.read()
        
        return self.process_response(data)


    def process_response(self, raw_response_xml):
        """
        Extract the data from raw response xml to a dictionary.
        """
#        raw_response_xml = """<?xml version="1.0" encoding="utf-8"?>
#        <createCustomerProfileTransactionResponse
#        xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
#        <messages>
#        <resultCode>Ok</resultCode>
#        <message>
#        <code>I00001</code>
#        <text>Successful.</text>
#        </message>
#        </messages>
#        <directResponse>1,1,1,This transaction has been
#        approved.,000000,Y,2000000001,INV000001,description of
#        transaction,10.95,CC,auth_capture,custId123,John,Doe,,123 Main
#        St.,Bellevue,WA,98004,USA,000-000-
#        0000,,mark@example.com,John,Doe,,123 Main
#        St.,Bellevue,WA,98004,USA,1.00,0.00,2.00,FALSE,PONUM000001,
#        D18EB6B211FE0BBF556B271FDA6F92EE,M,2,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#        </directResponse>
#        </createCustomerProfileTransactionResponse>"""
        
        
        e = ET.XML(raw_response_xml)
        d = self._recurive_parse(e)
        success = (d['messages']['result_code'].lower() == 'ok')
        
        d['result_code'] = d['messages']['result_code']
        d['message_code'] = d['messages']['message']['code']
        d['message_text'] = d['messages']['message']['text']
        
        return success, d
        
        
    def _recurive_parse(self, element):
        """
        Recurively parse the xml tree until we reach to the end node.
        
        example output:
        
        {    'customerProfileId': '4356210', 
            'customerPaymentProfileIdList': 
                {'numericString': '3831946'}, 
            'messages': 
                {'resultCode': 'Ok', 
                 'message': 
                     {'text': 'Successful.', 'code': 'I00001'}
                }, 
            'validationDirectResponseList': None,
            'customerShippingAddressIdList': None
        }
        """
        d = {}
        for sub_e in element:
            # remove the namespace in the tag
            # e.g.{AnetApi/xml/v1/schema/AnetApiSchema.xsd}resultCode
            name = re.sub(r'(\{.*?\})*(\w+)', r'\2', sub_e.tag)
            
            # convert camelCase to underscore
            name = re.sub('([A-Z])', lambda match: "_" + match.group(1).lower(), name)
            
            children = sub_e.getchildren()
            if not children:
                if d.has_key(name):
                    if not type(d[name]) is list:
                        d[name] = list([d[name]])
                    d[name].append(sub_e.text)
                else:
                    d[name] = sub_e.text
            else:
                d[name] = self._recurive_parse(sub_e)
        return d

class CIMCustomerProfile(CIMBase):
    def __init__(self, customer_profile_id=None):
        super(CIMCustomerProfile, self).__init__()
        
        self.customer_profile_id = customer_profile_id

    def create(self, **kwargs):
        """
        Create a customer profile on authorize.net
        Input fields:
            ref_id - optinal
            profile - required (either merchant_customer_id or email & description)
                merchant_customer_id
                description
                email
            payment_profiles - optinal
                customer_type - optinal
                bill_to
                    first_name, last_name, company, address, city, state, zip,
                    country, phone_number, fax_number, 
                payment
                    credit_card, card_number, expiration_date, card_code, bank_account
            account_type - optinal
            rounting_number
            account_number
            name_on_account
            echeck_type
            bank_name
                ship_to_list
                    first_name, last_name, company, address, city, state, zip, country
            phone_number
            fax_number
            validation_mode

        Output fields:
            ref_id
            customer_profile_id
            customer_payment_profile_id_list
            customer_shippingaddress_id_list
            validation_direct_response_list
            
        Example call:
           
        >>> from recurring_payments.authnet.cim import CIMCustomerProfile
        >>> cp = CIMCustomerProfile()
        >>> d = {'email': 'jqian@schipul.com', 'description': 'self registration',
                    'credit_card_info': {'card_number': '4111111111111111', 
                                        'expiration_date': '2015-12'}
                }
        >>> cp.create(**d)
        
        Sample request: 
        
        <?xml version="1.0" encoding="utf-8"?>
        <createCustomerProfileRequest xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
            <merchantAuthentication>
                <name>2J5ke9T6</name>
                <transactionKey>2CrW543gWMAs32vM</transactionKey>
            </merchantAuthentication>
            <profile>
                <description>self registration</description><email>jqian@schipul.com</email>
                <paymentProfiles><payment><creditCard><cardNumber>4111111111111111</cardNumber>
                <expirationDate>2015-12</expirationDate></creditCard></payment></paymentProfiles>
                </profile>
        </createCustomerProfileRequest>
        
        Sample response:
        
        <?xml version="1.0" encoding="utf-8"?>
        <createCustomerProfileResponse xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
        xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
            <messages>
                <resultCode>Ok</resultCode>
                <message>
                    <code>I00001</code>
                    <text>Successful.</text>
                </message>
            </messages>
            <customerProfileId>4356210</customerProfileId>
            <customerPaymentProfileIdList>
                <numericString>3831946</numericString>
            </customerPaymentProfileIdList>
            <customerShippingAddressIdList />
            <validationDirectResponseList />
        </createCustomerProfileResponse>

                    
        """
        root_name = 'createCustomerProfileRequest'
        xml_root = self.create_base_xml(root_name)
        ref_id = kwargs.get('ref_id', '')
        if ref_id:
            ET.SubElement(xml_root, 'refId')
        
        profile_node = self.create_profile_node(**kwargs)
        xml_root.append(profile_node)
        
        return self.process_request(xml_root)
    

    def delete(self, **kwargs):
        """
        Delete an existing customer profile along with all
        associated customer payment profiles and customer 
        shipping addresses.
        
        Input fields:
            refId - optional
            customer_profile_id
        """
        root_name = 'deleteCustomerProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        return self.process_request(xml_root)
        
        

    def get(self, **kwargs):
        """
        Get a customer profile from authorize.net
        
        Input fields:
            customer_profile_id
        
        Output fields:
        
        
        """
        root_name = 'getCustomerProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        return self.process_request(xml_root)
    
    def get_all(self, **kwargs):
        """
        Retrieve all existing customer profile Ids.
        
        Input fields:
            None
        Output fields:
            refId - optional
            ids
            
        """
        root_name = 'getCustomerProfileIdsRequest'
        xml_root = self.create_base_xml(root_name)
        return  self.process_request(xml_root)
              

    def update(self, **kwargs):
        """
        Update an existing customer profile on authorize.net.
        
        Input fields:
            refId - optional
            profile:
                merchant_customer_id - optional
                description - optional
                email - optional
                customer_profile_id
                
        Output fields:
            refId 
                
        """
        kwargs['mode'] = 'update'
        root_name = 'updateCustomerProfileRequest'
        xml_root = self.create_base_xml(root_name)
        ref_id = kwargs.get('ref_id', '')
        if ref_id:
            ET.SubElement(xml_root, 'refId')
        
        profile_node = self.create_profile_node(**kwargs)
        xml_root.append(profile_node)
        
        return self.process_request(xml_root)  
        

    def create_profile_node(self, **kwargs):
        customer_id = kwargs.get('customer_id', '')
        description = kwargs.get('description', '')
        email = kwargs.get('email', '')
        customer_profile_id = kwargs.get('customer_profile_id', '')
        mode = kwargs.get('mode', 'create')
        
        if mode == 'create':
            if not customer_id and not all([email, description]):
                raise AttributeError, "Either custom_id or email and description are required fields."
        else: # mode == 'update'
            if not customer_profile_id:
                raise AttributeError, "The customer_profile_id is a required field."
        
        profile_node = ET.Element("profile")
        if customer_id:
            customer_id_node = ET.SubElement(profile_node, 'merchantCustomerId')
            customer_id_node.text = customer_id
        if description:
            # profile description
            description_node = ET.SubElement(profile_node, 'description')
            description_node.text = description
        if email:
            email_node = ET.SubElement(profile_node, 'email')
            email_node.text = email
        if customer_profile_id:
            customer_profile_id_node = ET.SubElement(profile_node, 'customerProfileId')
            customer_profile_id_node.text = customer_profile_id
            
        if mode == 'create':
            # make payment_profiles node
            billing_info = kwargs.get('billing_info', '')
            credit_card_info= kwargs.get('credit_card_info', '')
            if billing_info or credit_card_info:
                payment_profiles_node = self.create_payment_profile_node('paymentProfiles', 
                                                                         billing_info, 
                                                                         credit_card_info)
                            
                
                profile_node.append(payment_profiles_node)
        
        return profile_node


class CIMCustomerPaymentProfile(CIMBase):
    def __init__(self, customer_profile_id, customer_payment_profile_id=None):
        super(CIMCustomerPaymentProfile, self).__init__()
        
        self.customer_profile_id = customer_profile_id
        self.customer_payment_profile_id = customer_payment_profile_id
        
        
    def create(self, **kwargs):
        """
        Create a new customer payment profile for an existing customer profile.
        
        Input fields:
            ref_id - optional
            customer_profile_id
            payment_profile:
                customer_type - optional
                bill_to:
                    first_name, last_name, company, address, city , state, zip, country, phone_number, fax_number
                payment:
                    credit_card
                        card_number, expiration_date, card_code
            validation_mode: none, testMode, liveMode
            
        Output fields:
            ref_id
            customer_payment_profile_id
            validation_direct_response
            
            
        Example call:
        d = {'ref_id': 222,
             'billing_info':{'first_name': 'John',
                             'last_name': 'Doe', 
                             'company': '',
                             'address': '123 Main St.', 
                             'city': 'Bellevue',
                             'state': 'WA',
                             'zip': '98004',
                             'country': 'USA',
                             'phoneNumber': '000-111-0000'
                             'faxNumber':''
                             },
             'credit_card_info': {
                                'card_number': '4111111111111111',
                                'expiration_date': '2023-12', 
                                '':
                                 }
        }
        
        cpp = CustomerPaymentProfile('4356210')
        cpp.create(**d)
        """
        
        if not self.customer_profile_id:
            raise AttributeError, "Missing customer_profile_id."
        
        root_name = 'createCustomerPaymentProfileRequest'
        xml_root = self.create_base_xml(root_name)
        ref_id = kwargs.get('ref_id', '')
        if ref_id:
            ET.SubElement(xml_root, 'refId')
            
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        billing_info = kwargs.get('billing_info', '')
        credit_card_info= kwargs.get('credit_card_info', '')
        if billing_info or credit_card_info:
            payment_profiles_node = self.create_payment_profile_node('paymentProfile', 
                                                                     billing_info, 
                                                                     credit_card_info)
            
            xml_root.append(payment_profiles_node)
        
        return  self.process_request(xml_root)
                
    
    def delete(self, **kwargs):
        """
        Delete a customer payment profile from an existing customer profile.
        
        Input fields:
            ref_id - optional
            customer_profile_id
            customer_payment_profile_id
            
       Output fields:
           ref_id - if included in the input
        
        """
        
        if not self.customer_profile_id or not self.customer_payment_profile_id:
            raise AttributeError, "Missing customer_profile_id or customer_payment_profile_id."
        
        root_name = 'deleteCustomerPaymentProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        customer_payment_profile_id_node = ET.SubElement(xml_root, 'customerPaymentProfileId')
        customer_payment_profile_id_node.text = self.customer_payment_profile_id
        
        return self.process_request(xml_root)
        
    
    def get(self, **kwargs):
        """
        retrieve a customer payment profile for an existing customer profile.
        
        Input fields:
            customer_profile_id
            customer_payment_profile_id
            
        Output fields:
            customer_payment_profile_id
            payment_profile:
                customer_type 
                bill_to:
                    first_name, last_name, company, address, city , state, zip, country, phone_number, fax_number
                payment:
                    credit_card
                        card_number, expiration_date, card_code
            
        """
        
        if not self.customer_profile_id or not self.customer_payment_profile_id:
            raise AttributeError, "Missing customer_profile_id or customer_payment_profile_id in input."
        
        root_name = 'getCustomerPaymentProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        customer_payment_profile_id_node = ET.SubElement(xml_root, 'customerPaymentProfileId')
        customer_payment_profile_id_node.text = self.customer_payment_profile_id
        
        return self.process_request(xml_root)
    
    def update(self, **kwargs):
        """
        Update a customer payment profile for an existing customer profile.
        CAUTION - "If some elements in this request are not submitted or are submitted with a blank value, the
        values in the original profile are removed on authorize.net.
        "
        
        Input fields:
            ref_id - optional
            customer_profile_id
            payment_profile:
                customer_payment_profile_id
                customer_type - optional
                bill_to:
                    first_name, last_name, company, address, city , state, zip, country, phone_number, fax_number
                payment:
                    credit_card
                        card_number, expiration_date, card_code
            validation_mode: none, testMode, liveMode
        
        Output fields:
            ref_id if included
            validation_direct_response
            
            
        """
        if not self.customer_profile_id or not self.customer_payment_profile_id:
            raise AttributeError, "Missing customer_profile_id or customer_payment_profile_id in input."
        
        root_name = 'updateCustomerPaymentProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        billing_info = kwargs.get('billing_info', '')
        credit_card_info= kwargs.get('credit_card_info', '')
        if billing_info or credit_card_info:
            payment_profiles_node = self.create_payment_profile_node('paymentProfile', 
                                                                     billing_info, 
                                                                     credit_card_info)
            
            customer_payment_profile_id_node = ET.SubElement(payment_profiles_node, 'customerPaymentProfileId')
            customer_payment_profile_id_node.text = self.customer_payment_profile_id
            
            xml_root.append(payment_profiles_node)
        
        return  self.process_request(xml_root)
                
        
    def validate(self, **kwargs):
        """
        Test if the last updated payment profile is valid.
        
        Input fields:
            customer_profile_id
            customer_payment_profile_id
            customer_shipping_address_id - optional
            card_code - optional
            validation_mode
            
        Output fields:
            
            
        """
        if not self.customer_profile_id or not self.customer_payment_profile_id:
            raise AttributeError, "Missing customer_profile_id or customer_payment_profile_id in input."
        
        root_name = 'validateCustomerPaymentProfileRequest'
        xml_root = self.create_base_xml(root_name)
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        customer_payment_profile_id_node = ET.SubElement(xml_root, 'customerPaymentProfileId')
        customer_payment_profile_id_node.text = self.customer_payment_profile_id
        
        customer_shipping_address_id = kwargs.get('customer_shipping_address_id')
        if customer_shipping_address_id:
            customer_shipping_address_id_node = ET.SubElement(xml_root, 'customerShippingAddressId')
            customer_shipping_address_id_node.text = customer_shipping_address_id
            
        validation_mode = kwargs.get('validation_mode')
        if validation_mode:
            validation_mode_node = ET.SubElement(xml_root, 'validationMode')
            validation_mode_node.text = validation_mode
        
        return self.process_request(xml_root)
        
        
class CIMCustomerProfileTransaction(CIMBase):
    def __init__(self, customer_profile_id, customer_payment_profile_id):
        super(CIMCustomerProfileTransaction, self).__init__()
        
        self.customer_profile_id = customer_profile_id
        self.customer_payment_profile_id = customer_payment_profile_id
        
        
    def create(self, **kwargs):
        """
        Create a payment transaction from an existing customer profile.
        for Authorization and Capture
        
        Input fields:
            ref_id  - optional
            transaction:
                profile_trans_auth_capture
                amount
                tax
                    amount, name, description
                shipping
                    amount, name, description
                duty
                    amount, name, description
                line_items
                    item_id, name, description, quantity, unit_price, taxable
                customer_profile_id
                customer_payment_profile_id
                customer_shipping_address_id
                returring_billing
                split_tender_id - conditional for partial authorization transaction
         
         Output fields:
               ref_id
               direct_response 
                        
        """
        if not self.customer_profile_id:
            raise AttributeError, "%s Missing customer_profile_id in input." % \
                                'createCustomerProfileTransactionRequest'
                                
        root_name = 'createCustomerProfileTransactionRequest'
        xml_root = self.create_base_xml(root_name)
        
        transaction_node = self.create_transaction_node(**kwargs)
        xml_root.append(transaction_node)
        
        # extra options
        extra_options_node = ET.SubElement(xml_root, 'extraOptions')
        extra_options_text = '&'.join(['%s=%s' % (x, y) for (x, y) in TRANSACTION_EXTRA_OPTIONS.items()])
        extra_options_node.text = extra_options_text
        
        return self.process_request(xml_root)
                
        
        
    def create_transaction_node(self, **kwargs):
        amount = kwargs.get('amount', 0)
        if amount <= 0:
            raise ValueError, '%s - the amount %.2f is not greater than 0.' % \
                               ('<createCustomerProfileTransactionRequest', amount)
        tax = kwargs.get('tax', '')
        shipping = kwargs.get('shipping')
        line_items_list = kwargs.get('line_items_list')
        customer_shipping_address_id = kwargs.get('customer_shipping_address_id')
        order = kwargs.get('order')
        tax_exempt = kwargs.get('tax_exempt')
        recurring_billing = kwargs.get('recurring_billing')
        card_code = kwargs.get('card_code')
        split_tender_id = kwargs.get('split_tender_id')
        
        transaction_node = ET.Element("transaction")
        trans_auth_capture_node = ET.SubElement(transaction_node, 'profileTransAuthCapture')
        
        
        # amount node
        amount_node = ET.SubElement(trans_auth_capture_node, "amount")
        amount_node.text = str(amount)
        
        # tax node       
        if tax and type(tax) is dict:
            tax = to_camel_case(tax)
            tax_node = ET.SubElement(trans_auth_capture_node, 'tax')
            tax_node = self.build_node_from_dict(tax_node, 
                                                 tax,
                                                 TAX_FIELDS)
            
        # tax node       
        if shipping and type(shipping) is dict:
            shipping = to_camel_case(shipping)
            shipping_node = ET.SubElement(trans_auth_capture_node, 'shipping')
            shipping_node = self.build_node_from_dict(shipping_node, 
                                                 shipping,
                                                 SHIPPING_FIELDS)
            
        # line items
        if line_items_list and type(line_items_list) is list:
            for line_items in line_items_list:
                if line_items and type(line_items) is dict:
                    line_items_node = ET.SubElement(trans_auth_capture_node, 'lineItems')
                    line_items_node = self.build_node_from_dict(line_items_node,
                                                                line_items,
                                                                LINE_ITEMS_FIELDS)
                    
        # customer profile id
        customer_profile_id_node = ET.SubElement(trans_auth_capture_node, "customerProfileId")
        customer_profile_id_node.text = self.customer_profile_id
        
        # customer payment profile id
        customer_payment_profile_id_node = ET.SubElement(trans_auth_capture_node, "customerPaymentProfileId")
        customer_payment_profile_id_node.text = self.customer_payment_profile_id
        
        # customer shipping address id
        if customer_shipping_address_id:
            customer_shipping_address_id_node = ET.SubElement(trans_auth_capture_node, "customerShippingAddressId")
            customer_shipping_address_id_node.text = customer_shipping_address_id
            
        # order
        if order and type(order) is dict:
            order = to_camel_case(order)
            order_node = ET.SubElement(trans_auth_capture_node, 'order')
            order_node = self.build_node_from_dict(order_node, 
                                                 order,
                                                 ORDER_FIELDS)
            
        # tax exempt
        if tax_exempt:
            tax_exempt_node = ET.SubElement(trans_auth_capture_node, "taxExempt")
            tax_exempt_node.text = tax_exempt
            
        # recurring billing
        if recurring_billing:
            recurring_billing_node = ET.SubElement(trans_auth_capture_node, "recurringBilling")
            recurring_billing_node.text = recurring_billing
            
        # card code
        if card_code:
            card_code_node = ET.SubElement(trans_auth_capture_node, "cardCode")
            card_code_node.text = card_code
            
        # split tender id
        if split_tender_id:
            split_tender_id_node = ET.SubElement(trans_auth_capture_node, "splitTenderId")
            split_tender_id_node.text = split_tender_id
                                               
            
        return transaction_node
    
    
class CIMHostedProfilePage(CIMBase):
    def __init__(self, customer_profile_id):
        super(CIMHostedProfilePage, self).__init__()
        
        self.customer_profile_id = customer_profile_id
        
    def get(self, **kwargs):
        """
        Get the token to initiate a request for direct access to the Authorize.Net hosted profile page.
        
        Input fields:
            customer_profile_id
            hosted_profile_settings - optional
                setting_name
                setting_value
                
        Output fields:
            token - if not used within 15 minutes of the original API call, this token expires.
            
        """
        if not self.customer_profile_id:
            raise AttributeError, "%s Missing customer_profile_id in input." % \
                                'getHostedProfilePageRequest'
                                
        root_name = 'getHostedProfilePageRequest'
        xml_root = self.create_base_xml(root_name)
        
        customer_profile_id_node = ET.SubElement(xml_root, 'customerProfileId')
        customer_profile_id_node.text = self.customer_profile_id
        
        hosted_profile_settings = kwargs.get('hosted_profile_settings')
        if hosted_profile_settings and type(hosted_profile_settings) is dict:
            hosted_profile_settings_node = ET.SubElement(xml_root, 'hostedProfileSettings')
            hosted_profile_settings = to_camel_case(hosted_profile_settings)
            for key in hosted_profile_settings.keys():
                setting_node = ET.SubElement(hosted_profile_settings_node, 'setting')
                hosted_profile_setting = {'settingName': key}
                setting_node = self.build_node_from_dict(setting_node, 
                                                         hosted_profile_setting, 
                                                         ('settingName', ))
                hosted_profile_setting = {'settingValue': hosted_profile_settings[key]}
                setting_node = self.build_node_from_dict(setting_node, 
                                                         hosted_profile_setting, 
                                                         ('settingName', 'settingValue'))
        #print ET.tostring(xml_root)
        return self.process_request(xml_root)
        
           

class CIMCustomerShippingAddress(CIMBase):
    def __init__(self, customer_profile_id):
        super(CIMCustomerShippingAddress, self).__init__()
        
        self.customer_profile_id = customer_profile_id
        
        
    def create(self, **kwargs):
        pass
    
    def delete(self, **kwargs):
        pass
    
    def get(self, **kwargs):
        pass
    
    def update(self, **kwargs):
        pass
    
    



from ordereddict import OrderedDict

from datetime import datetime, timedelta
from django.http import QueryDict
from django.forms import ChoiceField, MultiValueField
from django.forms.widgets import Widget, TextInput
from django.utils.safestring import mark_safe
from django.template.defaultfilters import date as date_filter

from tendenci.core.base.widgets import SplitDateTimeWidget
from widgets import UseCustomRegWidget


class Reg8nDtWidget(Widget):
    
    reg8n_dict = {
        'start_dt': '',
        'end_dt': '',
    }
    
    def render(self, name, value, attrs=None, choices=()):
        # rip prefix from name
        name_prefix = name.split('-')
        
        # This is a little hacky, and doesn't
        # compensate for dashes in a prefix
        # If you put dashes in a prefix, you're fired
        if len(name_prefix) > 2:
            # prefixes for form sets
            # Prefix = eloy
            # Field Name = eloy-0-fieldname .. eloy-N-fieldname
            prefix = '%s-%s' % (
                name_prefix[0],
                name_prefix[1]
            )
        else:
            # prefix for non-formsets
            # Prefix = eloy
            # Field Name = eloy-fieldname
            prefix = name_prefix[0]

        str_format_kwargs = []
        for k, v in self.reg8n_dict.items():
            # date field
            str_format_kwargs.append(SplitDateTimeWidget().render(
                '%s' % (k),  # name
                self.reg8n_dict.get(k),  # value
                {
                    'id': '%s' % (k)
                }  # id attribute
            ))
        
        # string format template
        html  = u"<div>%s to %s</div>" % tuple(str_format_kwargs)

        return mark_safe(html)


class Reg8nDtField(ChoiceField): 
    """
        Inherits from MultipleChoiceField and
        sets some default meta data
        note: This field injects 'data' into other form fields and
        causes formsets to consider new instances as 'modified' even 
        if a user introduces no changes to the form.
    """
    widget = Reg8nDtWidget

    def __init__(self, *args, **kwargs):
        super(Reg8nDtField, self).__init__(*args, **kwargs)
        self.build_widget_reg8n_dict()

    def build_widget_reg8n_dict(self, *args, **kwargs):
        """
        Build widget reg8n dictionary.
        Pass dictionary to widget.
        Please call() within form-init method
        """
        prefix = '%s-' % kwargs.get('prefix') if kwargs.get('prefix') else ''
        instance = kwargs.get('instance', None)
        initial = kwargs.get('initial') or {}
        
        today = datetime.today()
        one_hour = timedelta(hours=1)
        
        if instance:
            reg8n_dict = OrderedDict([
                ('%sstart_dt' % prefix, instance.start_dt),
                ('%send_dt' % prefix, instance.end_dt),
            ])
        else:
            reg8n_dict = OrderedDict([
                ('%sstart_dt' % prefix, initial.get('start_dt') or (today)),  # 2 hrs
                ('%send_dt' % prefix, initial.get('end_dt') or (today+(one_hour*3))),  # 3 hrs
            ])
        
        self.widget.reg8n_dict = reg8n_dict
        return reg8n_dict
    
    
class UseCustomRegField(MultiValueField):
    def __init__(self, required=True, widget=UseCustomRegWidget(attrs=None),
                label=None, initial=None, help_text=None):
        myfields = ()
        super(UseCustomRegField, self).__init__(myfields, required, widget,
                                          label, initial, help_text)
        
    def clean(self, value):
        return self.compress(value) 
        
    def compress(self, data_list):
        for i in range(0, len(data_list)):
            if type(data_list[i]) is bool:
                if data_list[i] == False:
                    data_list[i] = '0'
                else:
                    data_list[i] = '1'
            if data_list[i] == None:
                data_list[i] = ''
        
        if data_list:
            return ','.join(data_list)
        return None



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Meta'
        db.create_table('meta_meta', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
            ('keywords', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('canonical_url', self.gf('django.db.models.fields.CharField')(max_length=500, blank=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('meta', ['Meta'])


    def backwards(self, orm):
        
        # Deleting model 'Meta'
        db.delete_table('meta_meta')


    models = {
        'meta.meta': {
            'Meta': {'object_name': 'Meta'},
            'canonical_url': ('django.db.models.fields.CharField', [], {'max_length': '500', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['meta']



# python
import datetime
import re
import Image as Pil
import os
import mimetypes

# django
from django.http import HttpResponse, HttpResponseNotFound, Http404
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.decorators import login_required
from django.core.cache import cache
from django.template import RequestContext
from django.shortcuts import redirect
from django.conf import settings
from django.core.files.storage import default_storage

# local
from tendenci.core.base.cache import IMAGE_PREVIEW_CACHE
from tendenci.core.base.forms import PasswordForm
from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.site_settings.utils import get_setting


def image_preview(request, app_label, model, id,  size):
    """
        Grab all image link within a peice of content
        and generate thumbnails of largest image
    """    
    # get page object; protect size
    try:
        content_type = ContentType.objects.get(app_label=app_label, model=model)
        instance = content_type.get_object_for_this_type(id=id)
    except:
        return HttpResponseNotFound("Image not found.", mimetype="text/plain")
    
    keys = [settings.CACHE_PRE_KEY, IMAGE_PREVIEW_CACHE, model, str(instance.id), size]
    key = '.'.join(keys)
    response = cache.get(key)
    original_size = size
    
    if not response:
        from tendenci.core.base.utils import parse_image_sources, make_image_object_from_url, image_rescale

        # set sizes
        size_min = (30,30)
        size_cap = (512,512)
        
        size_tuple = size.split('x')
        if len(size_tuple) == 2: size = int(size_tuple[0]), int(size_tuple[1])
        else: size = int(size), int(size)

        if size > size_cap: size = size_cap

        image_urls = []

        image = Pil.new('RGBA',size_min)
    
        # find biggest image, dimension-wise
        for image_url in image_urls:
            image_candidate = make_image_object_from_url(image_url)
            if image_candidate:
                if image_candidate.size > image.size:
                    image = image_candidate

        if image.size[1] > size_min[1]:
        # rescale, convert-to true-colors; return response
    
            image = image_rescale(image, size)
            if image.mode != "RGB":
                image = image.convert("RGB")
    
            response = HttpResponse(mimetype='image/jpeg')
            
            image.save(response, "JPEG", quality=100)
            
            keys = [settings.CACHE_PRE_KEY, IMAGE_PREVIEW_CACHE, model, str(instance.id), size]
            key = '.'.join(keys)
            
            cache.set(key, response)
            return response
    
        else: # raise http 404 error (returns page not found)
            return HttpResponseNotFound("Image not found.", mimetype="text/plain")
    else:
        return response


def custom_error(request, template_name="500.html"):
    return render_to_response(template_name, {}, context_instance=RequestContext(request))


def plugin_static_serve(request, plugin, path, show_indexes=False):
    """
    Serve static files below a given point in the directory structure.

    To use, put a URL pattern such as::

        (r'^(?P<path>.*)$', 'django.views.static.serve', {'document_root' : '/path/to/my/files/'})

    in your URLconf. You must provide the ``document_root`` param. You may
    also set ``show_indexes`` to ``True`` if you'd like to serve a basic index
    of the directory.  This index view will use the template hardcoded below,
    but if you'd like to override it, you can create a template called
    ``static/directory_index.html``.
    """

    import mimetypes
    import os
    import posixpath
    import stat
    import urllib
    from email.Utils import parsedate_tz, mktime_tz

    from django.http import Http404, HttpResponse, HttpResponseRedirect, HttpResponseNotModified
    from django.utils.http import http_date
    from django.views.static import was_modified_since, directory_index

    from django.conf import settings

    document_root = os.path.join(settings.PROJECT_ROOT,'plugins',plugin,'media')

    # Clean up given path to only allow serving files below document_root.
    path = posixpath.normpath(urllib.unquote(path))
    path = path.lstrip('/')
    newpath = ''
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')

    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)
    fullpath = os.path.join(document_root, newpath)

    if os.path.isdir(fullpath):
        if show_indexes:
            return directory_index(newpath, fullpath)
        raise Http404("Directory indexes are not allowed here.")
    if not os.path.exists(fullpath):
        raise Http404('"%s" does not exist' % fullpath)

    # Respect the If-Modified-Since header.
    statobj = os.stat(fullpath)
    mimetype = mimetypes.guess_type(fullpath)[0] or 'application/octet-stream'
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
        return HttpResponseNotModified(mimetype=mimetype)
    contents = open(fullpath, 'rb').read()
    response = HttpResponse(contents, mimetype=mimetype)
    response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
    response["Content-Length"] = len(contents)
    return response

    
def clear_cache(request):
    cache.clear()
    return redirect('dashboard')


def memcached_status(request):
    try:
        import memcache
    except ImportError:
        raise Http404

    if not request.user.is_authenticated() and request.user.is_superuser:
        raise Http404

    # get first memcached URI
    m = re.match(
        "memcached://([.\w]+:\d+)", settings.CACHE_BACKEND
    )
    if not m:
        raise Http404

    host = memcache._Host(m.group(1))
    host.connect()
    host.send_cmd("stats")

    class Stats:
        pass

    stats = Stats()

    while 1:
        line = host.readline().split(None, 2)
        if line[0] == "END":
            break
        stat, key, value = line
        try:
            # convert to native type, if possible
            value = int(value)
            if key == "uptime":
                value = datetime.timedelta(seconds=value)
            elif key == "time":
                value = datetime.datetime.fromtimestamp(value)
        except ValueError:
            pass
        setattr(stats, key, value)

    host.close_socket()

    return render_to_response('base/memcached_status.html', {
            'stats': stats,
            'hit_rate': 100 * stats.get_hits / stats.cmd_get,
            'time': datetime.datetime.now(), # server time
    })


def feedback(request, template_name="base/feedback.html"):
    from tendenci.core.event_logs.models import EventLog
    if not request.user.profile.is_superuser:
        raise Http404
    EventLog.objects.log()
    return render_to_response(template_name, {}, context_instance=RequestContext(request))
    
def homepage(request, template_name="homepage.html"):
    from tendenci.core.event_logs.models import EventLog

    EventLog.objects.log()

    return render_to_response(template_name, {}, context_instance=RequestContext(request))


def robots_txt(request):
    options = ['base/robots_private.txt', 'base/robots_public.txt', 'robots.txt']
    template_name = "robots.txt"

    robots_setting = get_setting('site', 'global', 'robotstxt')
    if robots_setting in options:
        template_name = robots_setting

    return render_to_response(template_name, {}, context_instance=RequestContext(request), mimetype="text/plain")


def exception_test(request):
    raise Exception('Successfully raised the exception test. Boom.')
    return Http404


def file_display(request, file_path):
    base_name = os.path.basename(file_path)
    mime_type = mimetypes.guess_type(base_name)[0]

    if not mime_type:
        raise Http404

    if not default_storage.exists(file_path):
        raise Http404

    data = default_storage.open(file_path).read()

    response = HttpResponse(data, mimetype=mime_type)
    response['Content-Disposition'] = 'filename=%s' % base_name

    return response


@login_required
def password_again(request, template_name="base/password.html"):
    next = request.GET.get('next')

    if request.method == "POST":
        form = PasswordForm(request.POST, user=request.user)
        if form.is_valid():
            request.session['password_promt'] = True
            return redirect(next)
    else:
        form = PasswordForm(user=request.user)

    return render_to_response(template_name, {
        'next': next,
        'form': form,
    }, context_instance=RequestContext(request))




from django.conf import settings


# Maximum allowed length for field values.
FIELD_MAX_LENGTH = getattr(settings, "FORMS_BUILDER_FIELD_MAX_LENGTH", 2000)

# Maximum allowed length for field labels.
LABEL_MAX_LENGTH = getattr(settings, "FORMS_BUILDER_LABEL_MAX_LENGTH", 2000)

# Absolute path where files will be uploaded to.
UPLOAD_ROOT = getattr(settings, "FORMS_BUILDER_UPLOAD_ROOT", None)





from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import Group as Auth_Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

class Command(BaseCommand):
    # create an admin auth group if it doesn't exists, and assign all permissions 
    # (except 4 auth permission the auth_user, auth_groups..) to this group
    # command to run: python manage.py update_admin_group_perms
    def handle(self, *args, **options):
        out = ''
        if hasattr(settings, 'ADMIN_AUTH_GROUP_NAME'):
            name = settings.ADMIN_AUTH_GROUP_NAME
        else:
            name = 'Admin'
            
        try:
            auth_group = Auth_Group.objects.get(name=name)
        except Auth_Group.DoesNotExist:
            auth_group = Auth_Group(name=name)
            auth_group.save()
            #self.stdout.write('Successfully created an auth group "Admin".')
            out = 'Successfully created an auth group "Admin".\n'
        
        # assign permission to group, but exclude the auth content
        content_to_exclude = ContentType.objects.filter(app_label='auth')    
        permissions = Permission.objects.all().exclude(content_type__in=content_to_exclude)
        auth_group.permissions = permissions
        auth_group.save()
        
        #self.stdout.write('Successfully added all permissions to group "Admin".')
        out += 'Successfully added/updated all permissions to group "%s".' % name
        print out




from decimal import Decimal

from django import forms
from django.forms.formsets import BaseFormSet
from django.forms.util import ErrorList
from django.utils.translation import ugettext_lazy as _

from tendenci.core.site_settings.utils import get_setting
from tendenci.addons.events.registration.utils import can_use_pricing

class RegistrantBaseFormSet(BaseFormSet):
    """
    Extending the BaseFormSet to be able to add extra_params.
    note that extra_params does not consider conflicts in a single form's kwargs.
    """
    
    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                 initial=None, error_class=ErrorList, **kwargs):
        self.event = kwargs.pop('event')
        self.extra_params = kwargs.pop('extra_params', {})
        super(RegistrantBaseFormSet, self).__init__(data, files, auto_id, prefix,
                 initial, error_class)
                 
        # initialize internal variables
        self.pricings = {}
        self.enabled_pricings = []
        self.sets = []
        self.total_price = Decimal('0.00')
        
    def _construct_form(self, i, **kwargs):
        """
        Instantiates and returns the i-th form instance in a formset.
        """
        defaults = {
            'auto_id': self.auto_id,
            'prefix': self.add_prefix(i),
            'form_index': i,
        }
        
        for key in self.extra_params.keys():
            defaults[key] = self.extra_params[key]
        
        if self.data or self.files:
            defaults['data'] = self.data
            defaults['files'] = self.files
        if self.initial:
            try:
                defaults['initial'] = self.initial[i]
            except IndexError:
                pass
        
        # Allow extra forms to be empty.
        if i >= self.initial_form_count():
            defaults['empty_permitted'] = True
        defaults.update(kwargs)
        form = self.form(**defaults)
        self.add_fields(form, i)
        
        return form
        
    def set_pricing_groups(self):
        """
        Organize the forms based on pricings used
        """
        for i in range(0, self.total_form_count()):
            form = self.forms[i]
            pricing = form.saved_data['pricing']
            if pricing in self.pricings:
                self.pricings[pricing].append(form)
            else:
                self.pricings[pricing] = [form,]
    
    def set_sets(self):
        """
        Group each form to their corresponding set
        """
        for pricing in self.pricings.keys():
            for i in range(0, len(self.pricings[pricing])):
                form = self.pricings[pricing][i]
                if i % pricing.quantity == 0:
                    # initialize a new set with this form
                    new_set = [form] # we're assured this is always initialized.
                    self.sets.append(new_set)
                else:
                    new_set.append(form)
        
        # sort the sets based on first form label
        self.sets = sorted(self.sets, key=lambda s: s[0].get_form_label())
                    
    def get_sets(self):
        if not self.sets:
            self.set_sets()
        return self.sets
        
    def get_total_price(self):
        return self.total_price
        
    def get_user_list(self):
        """returns a list of user registrants"""
        users = []
        for i in range(0, self.total_form_count()):
            form = self.forms[i]
            user = form.get_user()
            if not user.is_anonymous():
                users.append(user)
        return users
    
    def clean(self):
        """
        Validate the set of registrants for all the pricings used.
        """
        
        if self.total_form_count()<1:
            raise forms.ValidationError(_('You must register at least 1 registrant per registration'))
        
        # organize pricing dict
        self.set_pricing_groups()
        
        # validate the reg quantity for each pricing
        for pricing in self.pricings.keys():
            # the registrant length must be divisible by the pricing's quantity
            if len(self.pricings[pricing]) % pricing.quantity != 0:
                raise forms.ValidationError(_("Please enter a valid number of registrants."))
        
        
        errors = []
        users = []
        
        # mark all validated prices is shared pricing
        shared_pricing = get_setting('module', 'events', 'sharedpricing')
        if shared_pricing:
            for pricing in self.pricings.keys():
                for i in range(0, len(self.pricings[pricing])):
                    form = self.pricings[pricing][i]
                    if i % pricing.quantity == 0:
                        user = form.get_user()
                        if can_use_pricing(self.event, user, pricing):
                            self.enabled_pricings.append(pricing)
                        
        # if all quantities are valid, update each form's corresponding price
        for pricing in self.pricings.keys():
            for i in range(0, len(self.pricings[pricing])):
                form = self.pricings[pricing][i]
                if i % pricing.quantity == 0:
                    price = pricing.price
                    
                    # first form of each set must be authorized for the pricing
                    user = form.get_user()
                    # take note of each invalid price but continue setting prices
                    
                    if shared_pricing:
                        if pricing not in self.enabled_pricings:
                            errors.append(forms.ValidationError(_("%(user)s is not authorized to use %(pricing)s" % {'user': user, 'pricing': pricing})))
                    else:
                        if not can_use_pricing(self.event, user, pricing):
                            errors.append(forms.ValidationError(_("%(user)s is not authorized to use %(pricing)s" % {'user': user, 'pricing': pricing})))
                    
                    if not user.is_anonymous():
                        # check if this user has already been used before
                        if user.pk in users:
                            errors.append(forms.ValidationError(_("%s can only be registered once per registration" % user)))
                        else:
                            # mark this pricing pair used
                            users.append(user.pk)
                else:
                    price = Decimal('0.00')
                
                # associate the price with the form
                form.set_price(price)
                
                # update the total price
                self.total_price += price
        
        # raise any errors found
        for error in errors:
            raise error
        



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Address'
        db.create_table('contacts_address', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('state', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('zipcode', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('country', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
        ))
        db.send_create_signal('contacts', ['Address'])

        # Adding model 'Phone'
        db.create_table('contacts_phone', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('number', self.gf('django.db.models.fields.CharField')(max_length=50, blank=True)),
        ))
        db.send_create_signal('contacts', ['Phone'])

        # Adding model 'Email'
        db.create_table('contacts_email', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
        ))
        db.send_create_signal('contacts', ['Email'])

        # Adding model 'URL'
        db.create_table('contacts_url', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
        ))
        db.send_create_signal('contacts', ['URL'])

        # Adding model 'Company'
        db.create_table('contacts_company', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
        ))
        db.send_create_signal('contacts', ['Company'])

        # Adding M2M table for field addresses on 'Company'
        db.create_table('contacts_company_addresses', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('company', models.ForeignKey(orm['contacts.company'], null=False)),
            ('address', models.ForeignKey(orm['contacts.address'], null=False))
        ))
        db.create_unique('contacts_company_addresses', ['company_id', 'address_id'])

        # Adding M2M table for field phones on 'Company'
        db.create_table('contacts_company_phones', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('company', models.ForeignKey(orm['contacts.company'], null=False)),
            ('phone', models.ForeignKey(orm['contacts.phone'], null=False))
        ))
        db.create_unique('contacts_company_phones', ['company_id', 'phone_id'])

        # Adding M2M table for field emails on 'Company'
        db.create_table('contacts_company_emails', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('company', models.ForeignKey(orm['contacts.company'], null=False)),
            ('email', models.ForeignKey(orm['contacts.email'], null=False))
        ))
        db.create_unique('contacts_company_emails', ['company_id', 'email_id'])

        # Adding M2M table for field urls on 'Company'
        db.create_table('contacts_company_urls', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('company', models.ForeignKey(orm['contacts.company'], null=False)),
            ('url', models.ForeignKey(orm['contacts.url'], null=False))
        ))
        db.create_unique('contacts_company_urls', ['company_id', 'url_id'])

        # Adding model 'Comment'
        db.create_table('contacts_comment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('contact', self.gf('django.db.models.fields.related.ForeignKey')(related_name='comments', to=orm['contacts.Contact'])),
            ('comment', self.gf('django.db.models.fields.TextField')()),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('contacts', ['Comment'])

        # Adding model 'Contact'
        db.create_table('contacts_contact', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('allow_anonymous_view', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('allow_user_view', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_member_view', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_anonymous_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_user_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_member_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(related_name='contacts_contact_creator', to=orm['auth.User'])),
            ('creator_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='contacts_contact_owner', to=orm['auth.User'])),
            ('owner_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('status_detail', self.gf('django.db.models.fields.CharField')(default='active', max_length=50)),
            ('guid', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('timezone', self.gf('timezones.fields.TimeZoneField')(default='US/Central', max_length=100)),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('middle_name', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('suffix', self.gf('django.db.models.fields.CharField')(max_length=5, blank=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('contacts', ['Contact'])

        # Adding M2M table for field addresses on 'Contact'
        db.create_table('contacts_contact_addresses', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('contact', models.ForeignKey(orm['contacts.contact'], null=False)),
            ('address', models.ForeignKey(orm['contacts.address'], null=False))
        ))
        db.create_unique('contacts_contact_addresses', ['contact_id', 'address_id'])

        # Adding M2M table for field phones on 'Contact'
        db.create_table('contacts_contact_phones', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('contact', models.ForeignKey(orm['contacts.contact'], null=False)),
            ('phone', models.ForeignKey(orm['contacts.phone'], null=False))
        ))
        db.create_unique('contacts_contact_phones', ['contact_id', 'phone_id'])

        # Adding M2M table for field emails on 'Contact'
        db.create_table('contacts_contact_emails', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('contact', models.ForeignKey(orm['contacts.contact'], null=False)),
            ('email', models.ForeignKey(orm['contacts.email'], null=False))
        ))
        db.create_unique('contacts_contact_emails', ['contact_id', 'email_id'])

        # Adding M2M table for field urls on 'Contact'
        db.create_table('contacts_contact_urls', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('contact', models.ForeignKey(orm['contacts.contact'], null=False)),
            ('url', models.ForeignKey(orm['contacts.url'], null=False))
        ))
        db.create_unique('contacts_contact_urls', ['contact_id', 'url_id'])

        # Adding M2M table for field companies on 'Contact'
        db.create_table('contacts_contact_companies', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('contact', models.ForeignKey(orm['contacts.contact'], null=False)),
            ('company', models.ForeignKey(orm['contacts.company'], null=False))
        ))
        db.create_unique('contacts_contact_companies', ['contact_id', 'company_id'])


    def backwards(self, orm):
        
        # Deleting model 'Address'
        db.delete_table('contacts_address')

        # Deleting model 'Phone'
        db.delete_table('contacts_phone')

        # Deleting model 'Email'
        db.delete_table('contacts_email')

        # Deleting model 'URL'
        db.delete_table('contacts_url')

        # Deleting model 'Company'
        db.delete_table('contacts_company')

        # Removing M2M table for field addresses on 'Company'
        db.delete_table('contacts_company_addresses')

        # Removing M2M table for field phones on 'Company'
        db.delete_table('contacts_company_phones')

        # Removing M2M table for field emails on 'Company'
        db.delete_table('contacts_company_emails')

        # Removing M2M table for field urls on 'Company'
        db.delete_table('contacts_company_urls')

        # Deleting model 'Comment'
        db.delete_table('contacts_comment')

        # Deleting model 'Contact'
        db.delete_table('contacts_contact')

        # Removing M2M table for field addresses on 'Contact'
        db.delete_table('contacts_contact_addresses')

        # Removing M2M table for field phones on 'Contact'
        db.delete_table('contacts_contact_phones')

        # Removing M2M table for field emails on 'Contact'
        db.delete_table('contacts_contact_emails')

        # Removing M2M table for field urls on 'Contact'
        db.delete_table('contacts_contact_urls')

        # Removing M2M table for field companies on 'Contact'
        db.delete_table('contacts_contact_companies')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 19, 15, 22, 2, 182857)'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 19, 15, 22, 2, 182768)'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contacts.address': {
            'Meta': {'object_name': 'Address'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'zipcode': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'})
        },
        'contacts.comment': {
            'Meta': {'object_name': 'Comment'},
            'comment': ('django.db.models.fields.TextField', [], {}),
            'contact': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'comments'", 'to': "orm['contacts.Contact']"}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'contacts.company': {
            'Meta': {'object_name': 'Company'},
            'addresses': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Address']", 'symmetrical': 'False', 'blank': 'True'}),
            'emails': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Email']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'phones': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Phone']", 'symmetrical': 'False', 'blank': 'True'}),
            'urls': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.URL']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'contacts.contact': {
            'Meta': {'object_name': 'Contact'},
            'addresses': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Address']", 'symmetrical': 'False', 'blank': 'True'}),
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'companies': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Company']", 'symmetrical': 'False', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'contacts_contact_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'emails': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Email']", 'symmetrical': 'False', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'middle_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'contacts_contact_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'phones': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.Phone']", 'symmetrical': 'False', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'suffix': ('django.db.models.fields.CharField', [], {'max_length': '5', 'blank': 'True'}),
            'timezone': ('timezones.fields.TimeZoneField', [], {'default': "'US/Central'", 'max_length': '100'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'urls': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contacts.URL']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'contacts.email': {
            'Meta': {'object_name': 'Email'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'contacts.phone': {
            'Meta': {'object_name': 'Phone'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'number': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'})
        },
        'contacts.url': {
            'Meta': {'object_name': 'URL'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'entities.entity': {
            'Meta': {'object_name': 'Entity'},
            'admin_notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'contact_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'email': ('django.db.models.fields.CharField', [], {'max_length': '120', 'blank': 'True'}),
            'entity_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'entity_parent_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'entity_type': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'summary': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'website': ('django.db.models.fields.CharField', [], {'max_length': '300', 'blank': 'True'})
        },
        'perms.objectpermission': {
            'Meta': {'object_name': 'ObjectPermission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']", 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True'})
        },
        'user_groups.group': {
            'Meta': {'object_name': 'Group'},
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_self_add': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_self_remove': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond_priority': ('django.db.models.fields.FloatField', [], {'default': '0', 'blank': 'True'}),
            'auto_respond_template': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email_recipient': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'entity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['entities.Entity']", 'null': 'True', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.User']", 'through': "orm['user_groups.GroupMembership']", 'symmetrical': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'group_permissions'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'show_as_option': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('tendenci.core.base.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'distribution'", 'max_length': '75', 'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'user_groups.groupmembership': {
            'Meta': {'unique_together': "(('group', 'member'),)", 'object_name': 'GroupMembership'},
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'member': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'group_member'", 'to': "orm['auth.User']"}),
            'owner_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'role': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['contacts']



# django
from datetime import datetime, timedelta
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, get_object_or_404, redirect, Http404
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponse
from django.contrib.auth.models import User
from django.db.models import Count, Q, get_app
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.translation import ugettext as _
from django.core.exceptions import ImproperlyConfigured
from django.contrib import messages
from django.conf import settings
# for password change
from django.views.decorators.csrf import csrf_protect

from djcelery.models import TaskMeta

from tendenci.core.base.decorators import ssl_required

from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.perms.utils import (has_perm, update_perms_and_save, get_notice_recipients, get_query_filters)
from tendenci.core.base.http import Http403
from tendenci.core.event_logs.models import EventLog
from tendenci.core.site_settings.utils import get_setting

# for avatar
from avatar.models import Avatar, avatar_file_path
from avatar.forms import PrimaryAvatarForm

# for group memberships
from tendenci.apps.user_groups.models import GroupMembership
from tendenci.apps.user_groups.forms import GroupMembershipEditForm

from tendenci.apps.profiles.models import Profile
from tendenci.apps.profiles.forms import (ProfileForm, ExportForm, UserPermissionForm, 
UserGroupsForm, ValidatingPasswordChangeForm, UserMembershipForm)
from tendenci.apps.profiles.tasks import ExportProfilesTask

try:
    notification = get_app('notifications')
except ImproperlyConfigured:
    notification = None

friends = False


@login_required
def index(request, username='', template_name="profiles/index.html"):
    """
    Show profile of username passed.  If no username is passed
    then redirect to username of person logged in.
    """

    if not username:
        return HttpResponseRedirect(reverse('profile', args=[request.user.username]))

    user_this = get_object_or_404(User, username=username)

    try:
        profile = user_this.get_profile()
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user_this)

    if not profile.allow_view_by(request.user):
        raise Http403

    # content counts
    content_counts = {'total': 0, 'invoice': 0}
    from tendenci.apps.invoices.models import Invoice
    inv_count = Invoice.objects.filter(Q(creator=user_this) | Q(owner=user_this), Q(bill_to_email=user_this.email)).count()
    if request.user.profile.is_superuser:    
        inv_count = Invoice.objects.filter(Q(creator=user_this) | Q(owner=user_this) | Q(bill_to_email=user_this.email)).count()
    content_counts['invoice'] = inv_count
    content_counts['total'] += inv_count

    # owners
    additional_owner_ids = ObjectPermission.objects.users_with_perms('profiles.change_profile', profile)
    additional_owners = []
    for id in additional_owner_ids:
        try:
            tmp_user = User.objects.get(pk=id)
            additional_owners.append(tmp_user)
        except User.DoesNotExist:
            pass
    if additional_owners:
        if profile.owner in additional_owners:
            additional_owners.remove(profile.owner)

    # group list
    group_memberships = user_this.group_member.all()

    memberships = user_this.memberships.filter(
                                    status=True,
                                    status_detail__in=['active', 'expired']
                                    )

    log_defaults = {
        'event_id': 125000,
        'event_data': '%s (%d) viewed by %s' % (profile._meta.object_name, profile.pk, request.user),
        'description': '%s viewed' % profile._meta.object_name,
        'user': request.user,
        'request': request,
        'instance': profile,
    }
    EventLog.objects.log(**log_defaults)

    state_zip = ' '.join([s for s in (profile.state, profile.zipcode) if s])
    city_state = ', '.join([s for s in (profile.city, profile.state) if s])
    city_state_zip = ', '.join([s for s in (profile.city, state_zip, profile.country) if s])

    can_edit = has_perm(request.user, 'profiles.change_profile', user_this)

    if not can_edit:
        can_edit = request.user == user_this

    return render_to_response(template_name, {
        'can_edit': can_edit,
        "user_this": user_this,
        "profile": profile,
        'city_state': city_state,
        'city_state_zip': city_state_zip,
        'content_counts': content_counts,
        'additional_owners': additional_owners,
        'group_memberships': group_memberships,
        'memberships': memberships,
        }, context_instance=RequestContext(request))


def search(request, template_name="profiles/search.html"):
    # check if allow anonymous user search
    allow_anonymous_search = get_setting('module', 'users', 'allowanonymoususersearchuser')
    allow_user_search = get_setting('module', 'users', 'allowusersearch')
    membership_view_perms = get_setting('module', 'memberships', 'memberprotection')

    if request.user.is_anonymous():
        if not allow_anonymous_search:
            raise Http403

    if request.user.is_authenticated():
        if not allow_user_search and not request.user.profile.is_superuser:
            raise Http403

    members = request.GET.get('members', None)
    query = request.GET.get('q', None)
    filters = get_query_filters(request.user, 'profiles.view_profile')
    profiles = Profile.objects.filter(Q(status=True), Q(status_detail="active"), Q(filters)).distinct()

    if query:
        profiles = profiles.filter(Q(status=True), Q(status_detail="active"), Q(user__first_name__icontains=query) | Q(user__last_name__icontains=query) | Q(user__email__icontains=query) | Q(user__username__icontains=query) | Q(display_name__icontains=query))

    if members:
        if not request.user.profile.is_superuser:
            if membership_view_perms == "private":
                profiles = profiles.filter(member_number="")
            elif membership_view_perms == "all-members" or membership_view_perms == "member-type":
                if request.user.profile and request.user.profile.is_member:
                    profiles = profiles.exclude(member_number="")
                else:
                    profiles = profiles.filter(member_number="")
            else:
                profiles = profiles.exclude(member_number="")
        else:
            profiles = profiles.exclude(member_number="")
    else:
        if not request.user.profile.is_superuser:
            if membership_view_perms == "private":
                    profiles = profiles.filter(member_number="")
            elif membership_view_perms == "all-members" or membership_view_perms == "member-type":
                if not request.user.profile or not request.user.profile.is_member:
                    profiles = profiles.filter(member_number="")

    profiles = profiles.order_by('user__last_name', 'user__first_name')

    EventLog.objects.log()
    return render_to_response(template_name, {'profiles': profiles, "user_this": None},
        context_instance=RequestContext(request))


@login_required
def add(request, form_class=ProfileForm, template_name="profiles/add.html"):
    if not has_perm(request.user,'profiles.add_profile'):raise Http403
    
    required_fields = get_setting('module', 'users', 'usersrequiredfields')
    if required_fields:
        required_fields_list = required_fields.split(',')
        required_fields_list = [field.strip() for field in required_fields_list]
    else:
        required_fields_list = None
    
    if request.method == "POST":
        form = form_class(request.POST, 
                          user_current=request.user,
                          user_this=None,
                          required_fields_list=required_fields_list)
        
        if form.is_valid():
            profile = form.save(request, None)
            new_user = profile.user
            
            # security_level
            if request.user.profile.is_superuser:
                security_level = form.cleaned_data['security_level']
                if security_level == 'superuser':
                    new_user.is_superuser = 1
                    new_user.is_staff = 1
                elif security_level == 'staff':
                    new_user.is_superuser = 0
                    new_user.is_staff = 1
                    
                else:
                    new_user.is_superuser = 0
                    new_user.is_staff = 0

                # set up user permission
                profile.allow_user_view, profile.allow_user_edit = False, False
                    
            else:
                new_user.is_superuser = 0
                new_user.is_staff = 0
                
            # interactive
            interactive = form.cleaned_data['interactive']
            try:
                interactive = int(interactive)
            except:
                interactive = 0

            new_user.is_active = interactive

            profile.save()
            new_user.save()

            ObjectPermission.objects.assign(new_user, profile)

            log_defaults = {
                'event_id' : 121000,
                'event_data': '%s (%d) added by %s' % (new_user._meta.object_name, new_user.pk, request.user),
                'description': '%s added' % new_user._meta.object_name,
                'user': request.user,
                'request': request,
                'instance': new_user,
            }
            EventLog.objects.log(**log_defaults)
            
            # send notification to administrators
            recipients = get_notice_recipients('module', 'users', 'userrecipients')
            if recipients:
                if notification:
                    extra_context = {
                        'object': profile,
                        'request': request,
                    }
                    notification.send_emails(recipients,'user_added', extra_context)
           
            return HttpResponseRedirect(reverse('profile', args=[new_user.username]))
    else:
        form = form_class(user_current=request.user,
                          user_this=None,
                          required_fields_list=required_fields_list)
    auto_pwd = request.POST.get('auto_pwd')
    return render_to_response(template_name, {'form':form, 'user_this':None,
                                              'required_fields_list': required_fields_list,
                                              'auto_pwd': auto_pwd
                                              }, 
        context_instance=RequestContext(request))
    

@login_required
def edit(request, id, form_class=ProfileForm, template_name="profiles/edit.html"):
    user_edit = get_object_or_404(User, pk=id)
    
    try:
        profile = Profile.objects.get(user=user_edit)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user_edit)
        
    if not profile.allow_edit_by(request.user): raise Http403
    
    required_fields = get_setting('module', 'users', 'usersrequiredfields')
    if required_fields:
        required_fields_list = required_fields.split(',')
        required_fields_list = [field.strip() for field in required_fields_list]
    else:
        required_fields_list = None
       
    if request.method == "POST":
        form = form_class(request.POST, 
                          user_current=request.user,
                          user_this=user_edit,
                          required_fields_list=required_fields_list,
                          instance=profile)
        
        if form.is_valid():
            # get the old profile, so we know what has been changed in admin notification
            old_user = User.objects.get(id=id)
            old_profile = Profile.objects.get(user=old_user)
            profile = form.save(request, user_edit)
           
            if request.user.profile.is_superuser:
                security_level = form.cleaned_data['security_level']
                
                if security_level == 'superuser':
                    user_edit.is_superuser = 1
                    user_edit.is_staff = 1
                    # remove them from auth_group if any - they don't need it
                    user_edit.groups = []
                elif security_level == 'staff':
                    user_edit.is_superuser = 0
                    user_edit.is_staff = 1
                else:
                    user_edit.is_superuser = 0
                    user_edit.is_staff = 0
                    # remove them from auth_group if any
                    user_edit.groups = []
                    
                    
                # set up user permission
                profile.allow_user_view, profile.allow_user_edit = False, False
                
            else:
                user_edit.is_superuser = 0
                user_edit.is_staff = 0
                
            # interactive
            interactive = form.cleaned_data['interactive']
            try:
                interactive = int(interactive)
            except:
                interactive = 0
            if interactive == 1:
                user_edit.is_active = 1
            else:
                user_edit.is_active = 0

            user_edit.save()
            profile.save()
            
            
            # notify ADMIN of update to a user's record
            if get_setting('module', 'users', 'userseditnotifyadmin'):
            #    profile_edit_admin_notify(request, old_user, old_profile, profile)
                # send notification to administrators
                recipients = get_notice_recipients('module', 'users', 'userrecipients')
                if recipients:
                    if notification:
                        extra_context = {
                            'old_user': old_user,
                            'old_profile': old_profile,
                            'profile': profile,
                            'request': request,
                        }
                        notification.send_emails(recipients,'user_edited', extra_context)
            

            log_defaults = {
                'event_id' : 122000,
                'event_data': '%s (%d) edited by %s' % (user_edit._meta.object_name, user_edit.pk, request.user),
                'description': '%s edited' % user_edit._meta.object_name,
                'user': request.user,
                'request': request,
                'instance': user_edit,
            }
            EventLog.objects.log(**log_defaults)
            return HttpResponseRedirect(reverse('profile', args=[user_edit.username]))
    else:
        if profile:
            form = form_class(user_current=request.user,
                          user_this=user_edit,
                          required_fields_list=required_fields_list,
                          instance=profile)
            
        else:
            form = form_class(user_current=request.user,
                          user_this=user_edit,
                          required_fields_list=required_fields_list)

    return render_to_response(template_name, {'user_this':user_edit, 'profile':profile, 'form':form,
                                              'required_fields_list':required_fields_list}, 
        context_instance=RequestContext(request))
    

    

def delete(request, id, template_name="profiles/delete.html"):
    user = get_object_or_404(User, pk=id)
    try:
        profile = Profile.objects.get(user=user)
    except:
        profile = None
    
    if not has_perm(request.user,'profiles.delete_profile',profile): raise Http403

    if request.method == "POST":
        recipients = get_notice_recipients('module', 'users', 'userrecipients')
        if recipients:
            if notification:
                extra_context = {
                    'profile': profile,
                    'request': request,
                }
                notification.send_emails(recipients,'user_deleted', extra_context)
        #soft delete
        #profile.delete()
        #user.delete()
        if profile:
            profile.status_detail = 'inactive'
            profile.save()
        user.is_active = False
        user.save()
        
        
        return HttpResponseRedirect(reverse('profile.search'))

    return render_to_response(template_name, {'user_this':user, 'profile': profile}, 
        context_instance=RequestContext(request))
    
@login_required
def edit_user_perms(request, id, form_class=UserPermissionForm, template_name="profiles/edit_perms.html"):
    user_edit = get_object_or_404(User, pk=id)
    try:
        profile = Profile.objects.get(user=user_edit)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user_edit)
   
    # for now, only admin can grant/remove permissions
    if not request.user.profile.is_superuser: raise Http403
    
    if request.method == "POST":
        form = form_class(request.POST, request.user, instance=user_edit)
    else:
        form = form_class(instance=user_edit)
    if form.is_valid():
        user_edit.is_superuser = form.cleaned_data['is_superuser']
        user_edit.user_permissions = form.cleaned_data['user_permissions']
        user_edit.save()
        EventLog.objects.log(instance=profile)

        return HttpResponseRedirect(reverse('profile', args=[user_edit.username]))
   
    return render_to_response(template_name, {'user_this':user_edit, 'profile':profile, 'form':form}, 
        context_instance=RequestContext(request))


def _get_next(request):
    """
    The part that's the least straightforward about views in this module is how they 
    determine their redirects after they have finished computation.

    In short, they will try and determine the next place to go in the following order:

    1. If there is a variable named ``next`` in the *POST* parameters, the view will
    redirect to that variable's value.
    2. If there is a variable named ``next`` in the *GET* parameters, the view will
    redirect to that variable's value.
    3. If Django can determine the previous page from the HTTP headers, the view will
    redirect to that previous page.
    """
    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', None)))
    if not next:
        next = request.path
    return next
   
@login_required
def change_avatar(request, id, extra_context={}, next_override=None):
    user_edit = get_object_or_404(User, pk=id)
    try:
        profile = Profile.objects.get(user=user_edit)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user_edit)
        
    #if not has_perm(request.user,'profiles.change_profile',profile): raise Http403
    if not profile.allow_edit_by(request.user): raise Http403
    
    avatars = Avatar.objects.filter(user=user_edit).order_by('-primary')
    if avatars.count() > 0:
        avatar = avatars[0]
        kwargs = {'initial': {'choice': avatar.id}}
    else:
        avatar = None
        kwargs = {}
    primary_avatar_form = PrimaryAvatarForm(request.POST or None, user=user_edit, **kwargs)
    if request.method == "POST":
        updated = False
        if 'avatar' in request.FILES:
            path = avatar_file_path(user=user_edit, 
                filename=request.FILES['avatar'].name)
            avatar = Avatar(
                user = user_edit,
                primary = True,
                avatar = path,
            )
            new_file = avatar.avatar.storage.save(path, request.FILES['avatar'])
            avatar.save()
            updated = True

            messages.add_message(
                request, messages.SUCCESS, _("Successfully uploaded a new avatar."))

        if 'choice' in request.POST and primary_avatar_form.is_valid():
            avatar = Avatar.objects.get(id=
                primary_avatar_form.cleaned_data['choice'])
            avatar.primary = True
            avatar.save()
            updated = True

            messages.add_message(
                request, messages.SUCCESS, _("Successfully updated your avatar."))

        if updated and notification:
            notification.send([request.user], "avatar_updated", {"user": user_edit, "avatar": avatar})
            #if friends:
            #    notification.send((x['friend'] for x in Friendship.objects.friends_for_user(user_edit)), "avatar_friend_updated", {"user": user_edit, "avatar": avatar})
        return HttpResponseRedirect(reverse('profile', args=[user_edit.username]))
        #return HttpResponseRedirect(next_override or _get_next(request))
    return render_to_response(
        'profiles/change_avatar.html',
        extra_context,
        context_instance = RequestContext(
            request,
            {'user_this': user_edit,
              'avatar': avatar, 
              'avatars': avatars,
              'primary_avatar_form': primary_avatar_form,
              'next': next_override or _get_next(request), }
        )
    )
    
@ssl_required    
@csrf_protect
@login_required
def password_change(request, id, template_name='registration/password_change_form.html',
                    post_change_redirect=None, password_change_form=ValidatingPasswordChangeForm):
    user_edit = get_object_or_404(User, pk=id)
    if post_change_redirect is None:
        post_change_redirect = reverse('profile', kwargs={'username': user_edit.username})
    if request.method == "POST":
        form = password_change_form(user=user_edit, data=request.POST)
        if request.user.profile.is_superuser:
            del form.fields['old_password']
        if form.is_valid():
            form.save()
            messages.add_message(
                request, messages.SUCCESS, _("Successfully updated your password."))
            return HttpResponseRedirect(post_change_redirect)
    else:
        form = password_change_form(user=user_edit)
        # an admin doesn't have to enter the old password
        if request.user.profile.is_superuser:
            del form.fields['old_password']
    return render_to_response(template_name, {
        'user_this': user_edit,
        'form': form,
    }, context_instance=RequestContext(request))

@login_required
def password_change_done(request, id, template_name='registration/password_change_done.html'):
    user_edit = get_object_or_404(User, pk=id)
    return render_to_response(template_name, {'user_this': user_edit},  context_instance=RequestContext(request))



### REPORTS ###########################################################################

def _user_events(from_date):
    return User.objects.all()\
                .filter(eventlog__create_dt__gte=from_date)\
                .annotate(event_count=Count('eventlog__pk'))

@staff_member_required
def user_activity_report(request, template_name='reports/user_activity.html'):
    now = datetime.now()
    users30days = _user_events(now-timedelta(days=10))
    users60days = _user_events(now-timedelta(days=60))
    users90days = _user_events(now-timedelta(days=90))
    # sort order of all fields for the upcoming response
    is_reverse = False
    is_ascending_username = True
    is_ascending_last_name = True
    is_ascending_first_name = True
    is_ascending_email = True
    is_ascending_events = True
    
    # get sort order
    sort = request.GET.get('sort', 'events')
    # Hande case if sort exists is one of the url parameters but blank
    if not sort:
        sort = 'events'
    
    if sort == 'username':
        users30days = users30days.order_by('username')
        users60days = users60days.order_by('username')
        users90days = users90days.order_by('username')
        is_ascending_username = False
    elif sort == '-username':
        users30days = users30days.order_by('-username')
        users60days = users60days.order_by('-username')
        users90days = users90days.order_by('-username')
        is_ascending_username = True
    elif sort == 'last_name':
        users30days = users30days.order_by('last_name')
        users60days = users60days.order_by('last_name')
        users90days = users90days.order_by('last_name')
        is_ascending_last_name = False
    elif sort == '-last_name':
        users30days = users30days.order_by('-last_name')
        users60days = users60days.order_by('-last_name')
        users90days = users90days.order_by('-last_name')
        is_ascending_last_name = True
    elif sort == 'first_name':
        users30days = users30days.order_by('first_name')
        users60days = users60days.order_by('first_name')
        users90days = users90days.order_by('first_name')
        is_ascending_first_name = False
    elif sort == '-first_name':
        users30days = users30days.order_by('-first_name')
        users60days = users60days.order_by('-first_name')
        users90days = users90days.order_by('-first_name')
        is_ascending_first_name = True
    elif sort == 'email':
        users30days = users30days.order_by('email')
        users60days = users60days.order_by('email')
        users90days = users90days.order_by('email')
        is_ascending_email = False
    elif sort == '-email':
        users30days = users30days.order_by('-email')
        users60days = users60days.order_by('-email')
        users90days = users90days.order_by('-email')
        is_ascending_email = True
    elif sort == 'events':
        users30days = users30days.order_by('-event_count')
        users60days = users60days.order_by('-event_count')
        users90days = users90days.order_by('-event_count')
        is_ascending_events = False
    elif sort == '-events':
        users30days = users30days.order_by('event_count')
        users60days = users60days.order_by('event_count')
        users90days = users90days.order_by('event_count')
        is_ascending_events = True
        
    # top 10 only
    users30days = users30days[:10]
    users60days = users60days[:10]
    users90days = users90days[:10]
    
    # Check for number sorting
    reverse = request.GET.get('reverse', 'False')
    if reverse == 'True':
        users30days = users30days[::-1]
        users60days = users60days[::-1]
        users90days = users90days[::-1]
        is_reverse = True
    else:
        is_reverse = False
    
    return render_to_response(template_name, {
        'users30days': users30days,
        'users60days': users60days,
        'users90days': users90days,
        'is_reverse': is_reverse,
        'is_ascending_username': is_ascending_username,
        'is_ascending_last_name': is_ascending_last_name,
        'is_ascending_first_name': is_ascending_first_name,
        'is_ascending_email': is_ascending_email,
        'is_ascending_events': is_ascending_events,
        }, context_instance=RequestContext(request))


@staff_member_required
def admin_users_report(request, template_name='reports/admin_users.html'):
    if not request.user.profile.is_superuser:
        raise Http403

    profiles = Profile.actives.filter(user__is_superuser=True).select_related()
    # sort order of all fields for the upcoming response
    is_ascending_id = True
    is_ascending_username = True
    is_ascending_last_name = True
    is_ascending_first_name = True
    is_ascending_email = True
    is_ascending_phone = True
    
    # get sort order
    sort = request.GET.get('sort', 'id')
    if sort == 'id':
        profiles = profiles.order_by('user__pk')
        is_ascending_id = False
    elif sort == '-id':
        profiles = profiles.order_by('-user__pk')
        is_ascending_id = True
    elif sort == 'username':
        profiles = profiles.order_by('user__username')
        is_ascending_username = False
    elif sort == '-username':
        profiles = profiles.order_by('-user__username')
        is_ascending_username = True
    elif sort == 'last_name':
        profiles = profiles.order_by('user__last_name')
        is_ascending_last_name = False
    elif sort == '-last_name':
        profiles = profiles.order_by('-user__last_name')
        is_ascending_last_name = True
    elif sort == 'first_name':
        profiles = profiles.order_by('user__first_name')
        is_ascending_first_name = False
    elif sort == '-first_name':
        profiles = profiles.order_by('-user__first_name')
        is_ascending_first_name = True
    elif sort == 'email':
        profiles = profiles.order_by('user__email')
        is_ascending_email = False
    elif sort == '-email':
        profiles = profiles.order_by('-user__email')
        is_ascending_email = True
    elif sort == 'phone':
        profiles = profiles.order_by('phone')
        is_ascending_phone = False
    elif sort == '-phone':
        profiles = profiles.order_by('-phone')
        is_ascending_phone = True
    
    return render_to_response(template_name, {
        'profiles': profiles,
        'is_ascending_id': is_ascending_id,
        'is_ascending_username': is_ascending_username,
        'is_ascending_last_name': is_ascending_last_name,
        'is_ascending_first_name': is_ascending_first_name,
        'is_ascending_email': is_ascending_email,
        'is_ascending_phone': is_ascending_phone,
        }, context_instance=RequestContext(request))
    
@staff_member_required
def user_access_report(request):
    now = datetime.now()
    logins_qs = EventLog.objects.filter(event_id=125200)
    
    total_users = User.objects.all().count()
    total_logins = logins_qs.count()
    
    day_logins = []
    for days in [30, 60, 90, 120, 182, 365]:
        count = logins_qs.filter(create_dt__gte=now-timedelta(days=days)).count()
        day_logins.append((days, count))
    
    return render_to_response('reports/user_access.html', {
                  'total_users': total_users,
                  'total_logins': total_logins,
                  'day_logins': day_logins,},  
                context_instance=RequestContext(request))

@login_required
def admin_list(request, template_name='profiles/admin_list.html'):
    # only admins can edit this list
    if not request.user.profile.is_superuser:
        raise Http403

    admins = Profile.actives.filter(user__is_superuser=True).select_related()
    
    return render_to_response(template_name, {'admins': admins},
                              context_instance=RequestContext(request))

@login_required
def users_not_in_groups(request, template_name='profiles/users_not_in_groups.html'):
    # superuser only
    if not request.user.profile.is_superuser:
        raise Http403

    users = []
    for user in User.objects.all():
        if not user.profile.get_groups():
            users.append(user)
    
    return render_to_response(template_name, {'users': users}, context_instance=RequestContext(request))

@login_required
def user_groups_edit(request, username, form_class=UserGroupsForm, template_name="profiles/add_delete_groups.html"):
    user = get_object_or_404(User, username=username)
    
    try:
        profile = Profile.objects.get(user=user)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user)
        
    if not profile.allow_edit_by(request.user):
        raise Http403
        
    if request.method == 'POST':
        form = form_class(user, request.user, request, request.POST)
        if form.is_valid():
            form.save()
            messages.add_message(request, messages.SUCCESS, 'Successfully edited groups for %s' % user.get_full_name())
            return HttpResponseRedirect("%s%s" % (reverse('profile', args=[user.username]),'#userview-groups'))
    else:
        form = form_class(user, request.user, request)

    return render_to_response(template_name, {
                            'form': form,
                            'user_this': user,
                            }, context_instance=RequestContext(request))

@login_required
def user_role_edit(request, username, membership_id, form_class=GroupMembershipEditForm, template_name="profiles/edit_role.html"):
    user = get_object_or_404(User, username=username)
    membership = get_object_or_404(GroupMembership, id=membership_id)
    
    try:
        profile = Profile.objects.get(user=user)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user)
    
    if not profile.allow_edit_by(request.user):
        raise Http403
    
    if not has_perm(request.user,'user_groups.view_group', membership.group):
        raise Http403
        
    if request.method == 'POST':
        form = form_class(request.POST, instance=membership)
        if form.is_valid():
            form.save()
            messages.add_message(request, messages.SUCCESS, 'Successfully edited membership for %s' % membership.group)
            return HttpResponseRedirect("%s%s" % (reverse('profile', args=[user.username]),'#userview-groups'))
    else:
        form = form_class(instance=membership)

    return render_to_response(template_name, {
                            'form': form,
                            'membership': membership,
                            }, context_instance=RequestContext(request))

@login_required
def user_membership_add(request, username, form_class=UserMembershipForm, template_name="profiles/add_membership.html"):
    user = get_object_or_404(User, username=username)
    
    try:
        profile = Profile.objects.get(user=user)
    except Profile.DoesNotExist:
        profile = Profile.objects.create_profile(user=user)
        
    if not request.user.profile.is_superuser:
        raise Http403
        
    if request.method == 'POST':
        form = form_class(request.POST)
        if form.is_valid():
            membership = form.save(commit=False)
            membership = update_perms_and_save(request, form, membership)
            messages.add_message(request, messages.SUCCESS, 'Successfully updated memberships for %s' % user.get_full_name())
            membership.populate_or_clear_member_id()
            return HttpResponseRedirect("%s%s" % (reverse('profile', args=[user.username]),'#userview-memberships'))
    else:
        form = form_class(initial={'user':user})

    return render_to_response(template_name, {
                            'form': form,
                            'user_this': user,
                            }, context_instance=RequestContext(request))

@login_required
def export(request, template_name="profiles/export.html"):
    """Create a csv file for all the users
    """

    if not request.user.profile.is_staff:
        raise Http404

    if request.method == 'POST':
        form = ExportForm(request.POST, user=request.user)
        if form.is_valid():
            if not settings.CELERY_IS_ACTIVE:
                task = ExportProfilesTask()
                response = task.run()
                return response
            else:
                task = ExportProfilesTask.delay()
                task_id = task.task_id
                return redirect('profile.export_status', task_id)
    else:
        form = ExportForm(user=request.user)
        
    return render_to_response(template_name, {
        'form':form,
        'user_this':None,
    }, context_instance=RequestContext(request))

def export_status(request, task_id, template_name="profiles/export_status.html"):
    try:
        task = TaskMeta.objects.get(task_id=task_id)
    except TaskMeta.DoesNotExist:
        task = None
        
    return render_to_response(template_name, {
        'task':task,
        'task_id':task_id,
        'user_this':None,
    }, context_instance=RequestContext(request))
    
def export_check(request, task_id):
    try:
        task = TaskMeta.objects.get(task_id=task_id)
    except TaskMeta.DoesNotExist:
        task = None
        
    if task and task.status == "SUCCESS":
        return HttpResponse("OK")
    else:
        return HttpResponse("DNE")


def export_download(request, task_id):
    try:
        task = TaskMeta.objects.get(task_id=task_id)
    except TaskMeta.DoesNotExist:
        task = None

    if task and task.status == "SUCCESS":
        return task.result
    else:
        raise Http404



from django.http import HttpResponse
from tastypie.utils import is_valid_jsonp_callback_value
from tastypie.utils.mime import determine_format, build_content_type
from tastypie.api import Api
from tastypie.exceptions import BadRequest
from tendenci.core.api_tasty.serializers import SafeSerializer

class SafeApi(Api):
    """
    Override to the tasty pie's default api to avoid errors with the
    optional libraries.
    """
    def top_level(self, request, api_name=None):
        """This is the same method in the Api model with the execption
        of the serializer used. This uses the SafeSerializer.
        """
        serializer = SafeSerializer()
        available_resources = {}
        
        if api_name is None:
            api_name = self.api_name
        
        for name in sorted(self._registry.keys()):
            available_resources[name] = {
                'list_endpoint': self._build_reverse_url("api_dispatch_list", kwargs={
                    'api_name': api_name,
                    'resource_name': name,
                }),
                'schema': self._build_reverse_url("api_get_schema", kwargs={
                    'api_name': api_name,
                    'resource_name': name,
                }),
            }
        
        desired_format = determine_format(request, serializer)
        options = {}
        
        if 'text/javascript' in desired_format:
            callback = request.GET.get('callback', 'callback')
        
            if not is_valid_jsonp_callback_value(callback):
                raise BadRequest('JSONP callback name is invalid.')
        
            options['callback'] = callback
        
        serialized = serializer.serialize(available_resources, desired_format, options)
        return HttpResponse(content=serialized, content_type=build_content_type(desired_format))



import os
import shutil
import sys
import boto
import urllib

from django.conf import settings
from django.core.cache import cache
from django.utils.importlib import import_module

from tendenci.core.theme.utils import get_theme_root, get_theme
from tendenci.apps.theme_editor.models import ThemeFileVersion
from tendenci.libs.boto_s3.utils import save_file_to_s3, read_theme_file_from_s3


template_directory = "/templates"
style_directory = "/media/css"

THEME_ROOT = get_theme_root()
TEMPLATES_ROOT = os.path.join(settings.PROJECT_ROOT, "templates")
ALLOWED_EXTENSIONS = (
    '.html',
    '.css',
    '.txt',
    '.js',
    '.po',
    '.less',
)

# At compile time, cache the directories to search.
fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
app_templates = {}
for app in settings.INSTALLED_APPS:
    try:
        mod = import_module(app)
    except ImportError, e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))
    template_dir = os.path.join(os.path.dirname(mod.__file__), 'templates')
    if os.path.isdir(template_dir):
        app_templates[app] = template_dir.decode(fs_encoding)


def copy(filename, path_to_file, full_filename, TO_ROOT=THEME_ROOT):
    """Copies a file and all associated directories into TO_ROOT
    """
    try:
        os.makedirs(os.path.join(TO_ROOT, "templates", path_to_file))
    except OSError:
        pass

    filecopy = os.path.join(TO_ROOT, "templates", path_to_file, filename)
    shutil.copy(full_filename, filecopy)

    # copy to s3
    if settings.USE_S3_THEME:
        if os.path.splitext(filename)[1] == '.html':
            public = False
        else:
            public = True
        dest_path = "/themes/%s" % filecopy
        save_file_to_s3(full_filename, dest_path=dest_path, public=public)


def qstr_is_dir(query_string, ROOT_DIR=THEME_ROOT):
    """
    Check to see if the query string is a directory or not
    """
    current_dir = os.path.join(ROOT_DIR, query_string)
    return os.path.isdir(current_dir)


def qstr_is_file(query_string, ROOT_DIR=THEME_ROOT):
    """
    Check to see if the query string is a directory or not
    """
    if settings.USE_S3_THEME:
        content = get_file_content(query_string)
        if content:
            return True

    current_file = os.path.join(ROOT_DIR, query_string)
    return os.path.isfile(current_file)


def get_dir_list(pwd, ROOT_DIR=THEME_ROOT):
    """
    Get a list of directories from within
    the theme folder based on the present
    working directory
    if the pwd is the None this will include a list of plugins in
    the dir_list.
    """
    dir_list = []
    current_dir = os.path.join(ROOT_DIR, pwd)
    if os.path.isdir(current_dir):
        item_list = os.listdir(current_dir)
        for item in item_list:
            current_item = os.path.join(current_dir, item)
            if os.path.isdir(current_item):
                dir_list.append(os.path.join(pwd, item))
    return sorted(dir_list)


def get_file_list(pwd, ROOT_DIR=THEME_ROOT):
    """
    Get a list of files from within
    the theme folder based on the present
    working directory
    """
    file_list = []
    others_list = []
    current_dir = os.path.join(ROOT_DIR, pwd)
    if os.path.isdir(current_dir):
        item_list = os.listdir(current_dir)
        for item in item_list:
            current_item = os.path.join(current_dir, item)
            if os.path.isfile(current_item):
                if os.path.splitext(current_item)[1] in ALLOWED_EXTENSIONS:
                    file_list.append(item)
                elif os.path.splitext(current_item)[1]:
                    others_list.append(item)
        return sorted(file_list), sorted(others_list)
    return file_list, others_list


def get_all_files_list(ROOT_DIR=THEME_ROOT):
    """
    Get a list of files and folders from within
    the theme folder
    """
    files_folders = {}
    root_dir = os.path.join(ROOT_DIR)

    start = root_dir.rfind(os.sep) + 1
    for path, dirs, files in os.walk(root_dir):
        subdir = {'contents': []}
        folders = path[start:].split(os.sep)
        for f in files:
            editable = False
            if os.path.splitext(os.path.join(path, f))[1] in ALLOWED_EXTENSIONS:
                editable = True
            subdir['contents'].append({'name': f, 'path': os.path.join(path[len(root_dir) + 1:], f), 'editable': editable})
        parent = reduce(dict.get, folders[:-1], files_folders)
        parent[folders[-1]] = subdir
        for parent in files_folders:
            subdir['contents'].append({'folder_path': path})

    if settings.USE_S3_THEME:
        s3_files_folders = {'contents': []}
        theme_folder = "%s/%s" % (settings.THEME_S3_PATH, get_theme())
        conn = boto.connect_s3(settings.AWS_ACCESS_KEY_ID,
                               settings.AWS_SECRET_ACCESS_KEY)
        bucket = conn.get_bucket(settings.AWS_STORAGE_BUCKET_NAME)

        for item in bucket.list(prefix=theme_folder):

            editable = False
            if os.path.splitext(item.name)[1] in ALLOWED_EXTENSIONS:
                editable = True

            file_path = item.name.replace(theme_folder, '').lstrip('/')
            path_split = file_path.split('/')
            splits = len(path_split)

            if splits == 1:
                s3_files_folders['contents'].append({
                        'name': path_split[0],
                        'path': file_path,
                        'editable': editable})
            elif splits == 2:
                if not path_split[0] in s3_files_folders:
                    s3_files_folders[path_split[0]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                s3_files_folders[path_split[0]]['contents'].append({
                        'name': path_split[1],
                        'path': file_path,
                        'editable': editable})
            elif splits == 3:
                if not path_split[0] in s3_files_folders:
                    s3_files_folders[path_split[0]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                if not path_split[1] in s3_files_folders[path_split[0]]:
                    s3_files_folders[path_split[0]][path_split[1]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                s3_files_folders[path_split[0]][path_split[1]]['contents'].append({
                        'name': path_split[2],
                        'path': file_path,
                        'editable': editable})
            elif splits == 4:
                if not path_split[0] in s3_files_folders:
                    s3_files_folders[path_split[0]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                if not path_split[1] in s3_files_folders[path_split[0]]:
                    s3_files_folders[path_split[0]][path_split[1]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                if not path_split[2] in s3_files_folders[path_split[0]][path_split[1]]:
                    s3_files_folders[path_split[0]][path_split[1]][path_split[2]] = {'contents': [{'folder_path': "/".join(path_split[:-1])}]}

                s3_files_folders[path_split[0]][path_split[1]][path_split[2]]['contents'].append({
                        'name': path_split[3],
                        'path': file_path,
                        'editable': editable})

        return {get_theme(): s3_files_folders}

    return files_folders


def get_file_content(file, ROOT_DIR=THEME_ROOT):
    """
    Get the content from the file that selected from
    the navigation
    """
    content = ''

    if settings.USE_S3_THEME:
        try:
            theme = get_theme()
            content = read_theme_file_from_s3(os.path.join(theme, file))
        except:
            pass

    if not content:
        current_file = os.path.join(ROOT_DIR, file)
        if os.path.isfile(current_file):
            fd = open(current_file, 'r')
            content = fd.read()
            fd.close()
    return content


def archive_file(request, relative_file_path, ROOT_DIR=THEME_ROOT):
    """
    Archive the file into the database if it is edited
    """
    file_path = os.path.join(ROOT_DIR, relative_file_path)
    if os.path.isfile(file_path):
        (file_dir, file_name) = os.path.split(file_path)
        fd = open(file_path, 'r')
        content = fd.read()
        fd.close()
        archive = ThemeFileVersion(file_name=file_name,
                                  content=content,
                                  relative_file_path=relative_file_path,
                                  author=request.user)
        archive.save()


def handle_uploaded_file(f, file_dir):
    filecopy = os.path.join(THEME_ROOT, file_dir, f.name)
    file_path = os.path.join(settings.PROJECT_ROOT, "themes", filecopy)

    destination = open(file_path, 'wb+')
    for chunk in f.chunks():
        destination.write(chunk)
    destination.close()

    # copy to s3
    if settings.USE_S3_THEME:
        if os.path.splitext(f.name)[1] == '.html':
            public = False
        else:
            public = True
        dest_path = "/themes/%s" % filecopy
        save_file_to_s3(file_path, dest_path=dest_path, public=public)

        cache_key = ".".join([settings.SITE_CACHE_KEY, 'theme', file_path[(file_path.find(THEME_ROOT)):]])
        cache.delete(cache_key)

        if hasattr(settings, 'REMOTE_DEPLOY_URL') and settings.REMOTE_DEPLOY_URL:
            urllib.urlopen(settings.REMOTE_DEPLOY_URL)



import uuid

from parse_uri import ParseUri

from django.db import models
from tendenci.apps.user_groups.models import Group
from tendenci.apps.user_groups.utils import get_default_group
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType

from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.categories.models import CategoryItem
from tendenci.core.site_settings.utils import get_setting
from tagging.fields import TagField
from tendenci.core.files.models import File, file_directory
from tendenci.core.perms.models import TendenciBaseModel
from tendenci.apps.stories.managers import StoryManager


class Story(TendenciBaseModel):
    """
    A Story is used across a site to add linked image content to a specific design area.
    The basic features of a Story include:

    - Title
    - Description (accepts HTML)
    - Image
    - Link

    Stories also include tags and a start and end time for automatic expiration.

    Stories use the Tendenci Base Model.
    """

    guid = models.CharField(max_length=40)
    title = models.CharField(max_length=200, blank=True)
    content = models.TextField(blank=True)
    syndicate = models.BooleanField(_('Include in RSS feed'))
    full_story_link = models.CharField(_('Full Story Link'), max_length=300, blank=True)
    link_title = models.CharField(_('Link Title'), max_length=200, blank=True)
    start_dt = models.DateTimeField(_('Start Date/Time'), null=True, blank=True)
    end_dt = models.DateTimeField(_('End Date/Time'), null=True, blank=True)
    expires = models.BooleanField(_('Expires'), default=True)
    ncsortorder = models.IntegerField(_('Order'), null=True, blank=True)
    image = models.ForeignKey('StoryPhoto',
        help_text=_('Photo that represents this story.'), null=True, default=None)
    group = models.ForeignKey(Group, null=True, default=get_default_group, on_delete=models.SET_NULL)
    tags = TagField(blank=True, default='')

    categories = generic.GenericRelation(CategoryItem,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = StoryManager()

    class Meta:
        permissions = (("view_story", "Can view story"),)
        verbose_name_plural = "stories"

    def __unicode__(self):
        return self.title

    @property
    def content_type(self):
        return 'stories'

    def photo(self):
        if self.image and self.image.file:
            return self.image.file

        return None

    def get_absolute_url(self):
        from django.core.urlresolvers import reverse
        url = reverse("story", args=[self.pk])
        if self.full_story_link:
            url = self.full_story_link
            parsed_url = ParseUri().parse(url)

            if not parsed_url.protocol:  # if relative URL
                url = '%s%s' % (get_setting('site', 'global', 'siteurl'), url)

        return url

    def save(self, *args, **kwargs):
        self.guid = self.guid or unicode(uuid.uuid1())
        photo_upload = kwargs.pop('photo', None)

        super(Story, self).save(*args, **kwargs)

        if photo_upload and self.pk:
            image = StoryPhoto(
                content_type=ContentType.objects.get_for_model(self.__class__),
                object_id=self.pk,
                creator=self.creator,
                creator_username=self.creator_username,
                owner=self.owner,
                owner_username=self.owner_username
                    )
            photo_upload.file.seek(0)
            image.file.save(photo_upload.name, photo_upload)  # save file row
            image.save()  # save image row

            if self.image:
                self.image.delete()  # delete image and file row
            self.image = image  # set image

            self.save()

        if self.ncsortorder is None:
            #Append
            try:
                last = Story.objects.all().exclude(pk=self.pk).order_by('-ncsortorder')[0]
                self.ncsortorder = int(last.ncsortorder) + 1
            except:
                #First row
                self.ncsortorder = 0

            self.save()

    @property
    def category_set(self):
        items = {}
        for cat in self.categories.select_related('category__name', 'parent__name'):
            if cat.category:
                items["category"] = cat.category
            elif cat.parent:
                items["sub_category"] = cat.parent
        return items


class StoryPhoto(File):
    pass




from django.db import models
from django.contrib.auth.models import User
from tendenci.core.files.models import File
from datetime import datetime

class BlogImport(models.Model):
    # who made the import
    author = models.ForeignKey(User)
    # when
    blog_import_date = models.DateTimeField(auto_now_add=True)
    # file
    blog = models.FileField(upload_to='blogimport')

class AssociatedFile(models.Model):
    post_id = models.IntegerField()
    file = models.ForeignKey(File)



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('tendenci.core.site_settings.views',
    url(r'^$', 'index', name="settings"),
    url(r'^(?P<scope>\w+)/(?P<scope_category>\w+)/$', 'list', name="settings.index"),
    url(r'^(?P<scope>\w+)/(?P<scope_category>\w+)/(?P<name>\w+)/$', 'single_setting', name="settings.single_setting"),
    url(r'^(?P<scope>\w+)/(?P<scope_category>\w+)/([#\w]+)$', 'list', name="setting.permalink"),
)
